# -*- coding: utf-8 -*-
import  html,requests,urllib.parse
requests.packages.urllib3.disable_warnings()
from typing import Union

def 网页_取Host(url: str) -> str:
    """
    从给定的URL中提取主机名。

    :param url: 需要提取主机名的URL字符串。
    :return: 提取出的主机名字符串。
    """
    url = url.strip(' ')  # 去除空格
    url = url if url.startswith('http') else 'http://' + url  # 补全Url
    return urllib.parse.urlsplit(url).hostname


def 网页_html字符解析(文本: str) -> str:
    """
    将特殊的HTML字符实体转换回普通字符。将&amp;这类字符转化成&

    :param 文本: 包含HTML字符实体的字符串。
    :return: 转换后的字符串。
    """
    return html.unescape(文本)


def 网页_取外网IP(返回地区: bool = False) -> tuple:
    """
    从多个在线服务中获取外网IP地址，并可选地返回相关的地区信息。

    :param 返回地区: 布尔值，指示是否返回地区信息。
    :return: 如果返回地区为真，则返回一个元组，包含IP和地区信息；否则仅返回IP地址。
    """
    header = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.152 Safari/537.36"}
    requests.packages.urllib3.disable_warnings()

    # 定义用于尝试获取IP的服务URL列表
    服务列表 = [
        {'id':1,"url": "https://ipservice.ws.126.net/locate/api/getLocByIp?callback=bowlder.cb._2", "start": 14, "end": -1},
        {'id':2,"url": "https://api.bilibili.com/x/web-interface/zone?jsonp=jsonp", "start": 0, "end": None}
    ]

    # 遍历服务列表，尝试获取IP和地区信息
    for 服务 in 服务列表:
        源码 = requests.get(服务["url"], verify=False, headers=header)
        结果 = eval(源码.text[服务["start"]:服务["end"]])

        if 服务["id"] == 1:
            if not 返回地区:
                return 结果['result']['ip'],''
            return 结果['result']['ip'], "{} {} {}".format(结果['result']['country'], 结果['result']['province'], 结果['result']['city'])

        if 服务["id"] == 2:
            if not 返回地区:
                return 结果['data']['addr'],''
            return 结果['data']['addr'], "{} {} {}".format(结果['data']['country'], 结果['data']['province'], 结果['data']['city'])
    # 如果所有尝试都失败，则根据返回地区参数返回相应的默认值
    return '', ''



class __网页返回类型:
    def __init__(self):
        self.源码 = ''
        self.字节集 = b'' #返回字节集,如图片,视频等
        self.cookie = {}
        self.协议头 = {}
        self.状态码 = 0
        self.requests = None
        self.json = {}

Union
def __参数处理(url: str, 方式: int = 0, 参数="", cookie={}, 协议头 = {}, 允许重定向: bool = True, 代理地址: str = None, 编码: str = None, 证书验证: bool = False, 上传文件: dict = None, 补全协议头: bool = True, json: dict = {}, 连接超时: int = 15, 读取超时: int = 15) -> dict():
    参数字典 = dict()
    _协议头 = dict()
    _cookie = dict()
    url = url.strip(' ')# 去除空格
    url = url if url.startswith('http') else 'http://' + url # 补全Url
    #协议头处理
    if type(协议头) == str: # 如果是文本则处理为字典格式
        协议头数组 = 协议头.split('\n')
        for x in 协议头数组:
            名称 = x[0:x.find(':')].strip(' ')
            值 = x[x.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
            if 名称 and 值:
                _协议头[名称] = 值
    else:
        _协议头 = 协议头

    if 补全协议头:
        host = urllib.parse.urlsplit(url).hostname
        if not 'Host' in _协议头:
            _协议头['Host'] = host
        if not 'Accept' in _协议头:
            _协议头['Accept'] = '*/*'
        if not 'Content-Type' in _协议头:
            _协议头['Content-Type'] = 'application/x-www-form-urlencoded'
        if not 'User-Agent' in _协议头:
            _协议头['User-Agent'] = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.152 Safari/537.36'
        if not 'Referer' in _协议头:
            _协议头['Referer'] = url

    #Cookie处理
    if type(cookie) == str:
        cookie数组 = cookie.split(';')
        for x in cookie数组:
            名称 = x[0:x.find("=")].strip(' ')
            值 = x[cookie.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
            if 名称 and 值:
                _cookie[名称] = 值
    else:
        _cookie = cookie
    #参数处理
    参数字典['url'] = url
    参数字典['verify'] = 证书验证
    参数字典['cookies'] = _cookie
    参数字典['headers'] = _协议头
    参数字典['allow_redirects'] = 允许重定向
    if 参数:
        if 方式 == 0:
            参数字典['params'] = 参数
        else:
            参数字典['data'] = 参数
    if json:
        参数字典['json'] = json
    if 上传文件:
        参数字典['files'] = 上传文件
    if 代理地址:
        参数字典['proxies'] = {"http": "http://" + 代理地址, "https": "https://" + 代理地址}
    if 连接超时 and 读取超时:
        参数字典['timeout'] = (连接超时, 读取超时)
    return 参数字典


def 网页_访问(url: str, 方式: int = 0, 参数="", cookie={}, 协议头 = {}, 允许重定向: bool = True, 代理地址: str = None, 编码: str = None, 证书验证: bool = False, 上传文件: dict = None, 补全协议头: bool = True, json: dict = {}, 连接超时: int = 15, 读取超时: int = 15,Class=requests) -> __网页返回类型:
    """
    从给定的URL访问网页，并返回响应内容。

    :param url: 链接,能自动补全http,去除首尾空格
    :param 方式: 请求方式（0.get, 1.post, 2.put, 3.delete, 4.head, 5.options）
    :param 参数: 请求参数（文本或字典）
    :param cookie: 请求cookie（文本或字典）
    :param 协议头: 请求头部信息（文本或字典）
    :param 允许重定向: 是否允许自动重定向
    :param 代理地址: 代理服务器地址 账号:密码@IP:端口  或  IP:端口
    :param 编码: 响应内容的编码方式  utf8,gbk·······
    :param 证书验证: 是否验证HTTPS证书
    :param 上传文件: 上传的文件信息 {'upload': ('code.png', 图片字节集, 'image/png')}
    :param 补全协议头: 是否补全标准协议头
    :param json: JSON格式的参数
    :param 连接超时: 连接超时时间
    :param 读取超时: 读取超时时间
    :return: 返回网页对象
    """
    参数字典 = __参数处理(url, 方式, 参数, cookie, 协议头, 允许重定向, 代理地址, 编码, 证书验证, 上传文件, 补全协议头,json,连接超时, 读取超时)

    网页 = __网页返回类型()
    请求方式映射 = {
        0: Class.get,
        1: Class.post,
        2: Class.put,
        3: Class.delete,
        4: Class.head,
        5: Class.options
    }
    if 方式 in 请求方式映射:
        请求方法 = 请求方式映射[方式]
        try:
            网页对象 = 请求方法(**参数字典)
            if 编码:
                网页对象.encoding = 编码
            网页.requests = 网页对象
            网页.源码 = 网页对象.text
            网页.cookie = dict(网页对象.cookies)
            网页.状态码 = 网页对象.status_code
            网页.协议头 = 网页对象.headers
            网页.字节集 = 网页对象.content
            网页.json = 网页对象.json()
        except Exception as e:
            网页.json = {}
    return 网页


class 网页_会话:
    "自动处理Cookie"

    def __init__(self,重连次数: int = 0):
        """
        初始化网络请求会话，并设置重连次数。

        :param 重连次数: HTTP/HTTPS 请求的最大重试次数。
        """
        self.requests = requests.session()
        if 重连次数:
            self.requests.mount('http://', HTTPAdapter(max_retries=重连次数))
            self.requests.mount('https://', HTTPAdapter(max_retries=重连次数))


    def 访问(self,url: str, 方式: int = 0, 参数="", cookie={}, 协议头={}, 允许重定向: bool = True, 代理地址: str = None, 编码: str = None, 证书验证: bool = False, 上传文件: dict = None, 补全协议头: bool = True, json: dict = {}, 连接超时: int = 15, 读取超时: int = 15):
        """
        从给定的URL访问网页，并返回响应内容。

        :param url: 链接,能自动补全http,去除首尾空格
        :param 方式: 请求方式（0.get, 1.post, 2.put, 3.delete, 4.head, 5.options）
        :param 参数: 请求参数（文本或字典）
        :param cookie: 请求cookie（文本或字典）
        :param 协议头: 请求头部信息（文本或字典）
        :param 允许重定向: 是否允许自动重定向
        :param 代理地址: 代理服务器地址 账号:密码@IP:端口  或  IP:端口
        :param 编码: 响应内容的编码方式  utf8,gbk·······
        :param 证书验证: 是否验证HTTPS证书
        :param 上传文件: 上传的文件信息 {'upload': ('code.png', 图片字节集, 'image/png')}
        :param 补全协议头: 是否补全标准协议头
        :param json: JSON格式的参数
        :param 连接超时: 连接超时时间
        :param 读取超时: 读取超时时间
        :return: 返回网页对象
        """
        return 网页_访问(url, 方式, 参数, cookie, 协议头, 允许重定向, 代理地址, 编码, 证书验证, 上传文件, 补全协议头,json,连接超时, 读取超时,Class=self.requests)


# -*- coding: utf-8 -*-
import re,random,json




def 文本_JSON转字典(内容: str) -> dict:
    """
    将JSON格式的字符串转换为Python字典。

    :param 内容: JSON格式的字符串。
    :return: 对应的Python字典。
    """
    return json.loads(内容)


def 文本_对象转文本(对象) -> str:
    """
    将任何Python对象转换为其文本表示形式。

    :param 对象: 任何Python对象。
    :return: 对象的文本表示形式。
    """
    return repr(对象)


def 文本_取左边(原文本: str, 要取出的数量: int) -> str:
    """
    从原文本中取出指定数量的字符，从左边开始。

    :param 原文本: 要进行操作的字符串。
    :param 要取出的数量: 从左边开始要取出的字符数量。
    :return: 取出的字符串。
    """
    return 原文本[:要取出的数量]


def 文本_取右边(原文本: str, 要取出的数量: int) -> str:
    """
    从原文本中取出指定数量的字符，从右边开始。

    :param 原文本: 要进行操作的字符串。
    :param 要取出的数量: 从右边开始要取出的字符数量。
    :return: 取出的字符串。
    """
    return 原文本[-要取出的数量:]


def 文本_取文本左边(原文本: str, 指定的文本: str) -> str:
    """
    从原文本中找到指定文本的位置，并返回其左边的部分。

    :param 原文本: 要进行操作的字符串。
    :param 指定的文本: 需要定位的字符串。
    :return: 指定文本左边的部分。如果未找到指定文本，则返回空字符串。
    """
    索引 = 原文本.find(指定的文本)
    return 原文本[:索引] if 索引 != -1 else ''


def 文本_取文本右边(原文本: str, 指定的文本: str) -> str:
    """
    从原文本中找到指定文本的最后出现位置，并返回其右边的部分。

    :param 原文本: 要进行操作的字符串。
    :param 指定的文本: 需要定位的字符串。
    :return: 指定文本右边的部分。如果未找到指定文本，则返回空字符串。
    """
    索引 = 原文本.rfind(指定的文本)
    return 原文本[索引 + len(指定的文本):] if 索引 != -1 else ''


import json

def 文本_JSON解析(内容, 中文解码: bool = False) -> str:
    """
    将给定的内容转换为 JSON 字符串。如果启用中文解码，汉字不会被转换为 Unicode 编码。

    :param 内容: 要转换为 JSON 字符串的内容。
    :param 中文解码: 指定是否使用中文解码，默认为 False。
    :return: JSON 格式的字符串。
    """
    return json.dumps(内容, ensure_ascii=not 中文解码)


def 文本_寻找文本(原文本: str, 寻找的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> int:
    """
    在原文本中寻找指定的文本，返回其位置（从 0 开始）。如果未找到，返回 -1。

    :param 原文本: 要搜索的字符串。
    :param 寻找的文本: 需要查找的字符串。
    :param 开始的位置: 搜索的起始位置，默认为 0。
    :param 结束的位置: 搜索的结束位置，默认为 0，表示搜索到字符串的末尾。
    :return: 寻找的文本在原文本中的位置，如果未找到则返回 -1。
    """
    return 原文本.find(寻找的文本, 开始的位置, 结束的位置 or None)


def 文本_倒找文本(原文本: str, 寻找的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> int:
    """
    类似于 find() 方法，但从原文本的右边开始查找指定的文本。

    :param 原文本: 要搜索的字符串。
    :param 寻找的文本: 需要查找的字符串。
    :param 开始的位置: 搜索的起始位置，默认为 0。
    :param 结束的位置: 搜索的结束位置，默认为 0，表示搜索到字符串的末尾。
    :return: 寻找的文本在原文本中的位置，如果未找到则返回 -1。
    """
    return 原文本.rfind(寻找的文本, 开始的位置, 结束的位置 or None)


def 文本_取出中间文本(原文本: str, 前面的文本: str, 后面的文本: str, 开始的位置: int = 0) -> str:
    """
    在原文本中查找并返回前面的文本和后面的文本之间的部分。

    :param 原文本: 要搜索的字符串。
    :param 前面的文本: 定位起始位置的字符串。
    :param 后面的文本: 定位结束位置的字符串。
    :param 开始的位置: 搜索的起始位置，默认为 0。
    :return: 位于前面的文本和后面的文本之间的字符串。如果未找到，则返回空字符串。
    """
    位置_前 = 原文本.find(前面的文本, 开始的位置)
    位置_后 = 原文本.find(后面的文本, 位置_前 + len(前面的文本))
    if 位置_前 != -1 and 位置_后 != -1:
        return 原文本[位置_前 + len(前面的文本):位置_后]
    else:
        return ''


def 文本_倒取出中间文本(原文本: str, 右边的文本: str, 左边的文本: str, 开始的位置: int = -1) -> str:
    """
    从原文本的右边开始，查找并返回左边的文本和右边的文本之间的部分。

    :param 原文本: 要搜索的字符串。
    :param 右边的文本: 定位结束位置的字符串。
    :param 左边的文本: 定位起始位置的字符串。
    :param 开始的位置: 搜索的起始位置，默认为 -1，表示从字符串末尾开始。
    :return: 位于左边的文本和右边的文本之间的字符串。如果未找到，则返回空字符串。
    """
    位置_右 = 原文本[:开始的位置].rfind(右边的文本)
    位置_左 = 原文本[:位置_右].rfind(左边的文本)
    if 位置_右 != -1 and 位置_左 != -1:
        return 原文本[位置_左 + len(左边的文本):位置_右]
    else:
        return ''

def 文本_倒取中间_批量(原文本: str, 文本A: str, 文本B: str) -> list:
    """
    在原文本中以相反的顺序查找并返回所有文本B和文本A之间的部分。

    :param 原文本: 要搜索的字符串。
    :param 文本A: 定位结束位置的字符串。
    :param 文本B: 定位起始位置的字符串。
    :return: 一个列表，包含所有位于文本B和文本A之间的字符串，以相反的顺序返回。如果未找到，则返回空列表。
    """
    搜索位置 = 0
    结果列表 = []
    while True:
        位置_B = 原文本.find(文本B, 搜索位置)
        if 位置_B != -1:
            位置_A = 原文本.find(文本A, 位置_B + len(文本B))
            if 位置_A != -1:
                取出的文本 = 原文本[位置_B + len(文本B):位置_A]
                if 取出的文本:
                    结果列表.append(取出的文本)
                搜索位置 = 位置_A + len(文本A)
            else:
                break
        else:
            break
    结果列表.reverse()
    return 结果列表



def 文本_取中间_批量(原文本: str, 前面的文本: str, 后面的文本: str) -> list:
    """
    在原文本中查找并返回所有前面的文本和后面的文本之间的部分。

    :param 原文本: 要搜索的字符串。
    :param 前面的文本: 定位起始位置的字符串。
    :param 后面的文本: 定位结束位置的字符串。
    :return: 一个列表，包含所有位于前面的文本和后面的文本之间的字符串。如果未找到，则返回空列表。
    """
    搜索位置 = 0
    结果列表 = []
    while True:
        位置_前 = 原文本.find(前面的文本, 搜索位置)
        if 位置_前 != -1:
            位置_后 = 原文本.find(后面的文本, 位置_前 + len(前面的文本))
            if 位置_后 != -1:
                搜索位置 = 位置_前 + len(前面的文本)
                取出的文本 = 原文本[搜索位置:位置_后]
                if 取出的文本:
                    结果列表.append(取出的文本)
                搜索位置 = 位置_后
            else:
                break
        else:
            break
    return 结果列表



def 文本_子文本替换(原文本: str, 要替换的文本: str, 用作替换的文本: str, 替换的次数: int = -1) -> str:
    """
    在原文本中替换指定的子文本。

    :param 原文本: 要进行替换操作的字符串。
    :param 要替换的文本: 需要被替换的子文本。
    :param 用作替换的文本: 用于替换的子文本。
    :param 替换的次数: 指定替换操作的次数，默认为 -1，表示替换所有匹配项。
    :return: 替换后的字符串。
    """
    return 原文本.replace(要替换的文本, 用作替换的文本, 替换的次数)


def 文本_删除空行(原文本: str) -> str:
    """
    删除文本中的所有空行。

    :param 原文本: 原始的字符串。
    :return: 删除空行后的字符串。
    """
    # 按行分割文本，去除空行或只包含空白字符的行
    处理后的行 = [行 for 行 in 原文本.split('\n') if 行.strip()]
    # 将非空行重新组合成字符串
    处理后的文本 = '\n'.join(处理后的行)
    return 处理后的文本

def 文本_只取字母(原文本: str) -> str:
    """
    从文本中提取所有的字母。

    :param 原文本: 原始文本字符串。
    :return: 所有字母组成的字符串。
    """
    return ''.join(re.findall(r'[A-Za-z]', 原文本))

def 文本_只取数字(原文本: str) -> str:
    """
    从文本中提取所有的数字。

    :param 原文本: 原始文本字符串。
    :return: 所有数字组成的字符串。
    """
    return ''.join(re.findall(r'\d', 原文本))

def 文本_只取符号(原文本: str) -> str:
    """
    从文本中提取所有的符号（排除字母、数字和汉字）。

    :param 原文本: 原始文本字符串。
    :return: 所有符号组成的字符串。
    """
    # 排除字母、数字、汉字以及空白字符
    return ''.join(re.findall(r'[^\w\s\u4e00-\u9fff]', 原文本))

def 文本_只取大写字母(原文本: str) -> str:
    """
    从文本中提取所有的大写字母。

    :param 原文本: 原始文本字符串。
    :return: 所有大写字母组成的字符串。
    """
    return ''.join(re.findall(r'[A-Z]', 原文本))

def 文本_只取小写字母(原文本: str) -> str:
    """
    从文本中提取所有的小写字母。

    :param 原文本: 原始文本字符串。
    :return: 所有小写字母组成的字符串。
    """
    return ''.join(re.findall(r'[a-z]', 原文本))

def 文本_只取汉字(原文本: str) -> str:
    """
    从文本中提取所有汉字。

    :param 原文本: 原始文本字符串。
    :return: 所有汉字组成的字符串。
    """
    # 使用正则表达式匹配所有汉字
    汉字列表 = re.findall(r'[\u4e00-\u9fff]+', 原文本)
    return ''.join(汉字列表)

def 文本_去重复文本(原文本: str) -> str:
    """
    去除文本中所有重复出现的字符。

    :param 原文本: 原始文本字符串。
    :return: 去重后的文本字符串。
    """
    已出现字符集合 = set()
    去重文本 = []

    for 字符 in 原文本:
        if 字符 not in 已出现字符集合:
            去重文本.append(字符)
            已出现字符集合.add(字符)

    return ''.join(去重文本)

def 文本_分割文本(原文本: str, 分割标识: str, 分割次数: int = -1) -> list:
    """
    使用指定的分割标识来分割原文本。如果指定了分割次数，返回分割次数加一的列表。

    :param 原文本: 要分割的字符串。
    :param 分割标识: 用于分割的标识。
    :param 分割次数: 分割操作的次数，默认为 -1，表示分割所有可能的部分。
    :return: 分割后的字符串列表。
    """
    return 原文本.split(分割标识, 分割次数)


def 文本_换行分割(原文本: str, 保留换行: bool = False) -> list:
    """
    以换行符（\r, \n, \r\n）分割原文本。可以选择是否保留换行符。

    :param 原文本: 要分割的字符串。
    :param 保留换行: 是否保留换行符，默认为 False。
    :return: 分割后的字符串列表。
    """
    return 原文本.splitlines(保留换行)


def 文本_到小写(原文本: str) -> str:
    """
    将原文本转换为小写。

    :param 原文本: 要转换的字符串。
    :return: 转换为小写的字符串。
    """
    return 原文本.casefold()


def 文本_到大写(原文本: str) -> str:
    """
    将原文本转换为大写。

    :param 原文本: 要转换的字符串。
    :return: 转换为大写的字符串。
    """
    return 原文本.upper()


import re

def 文本_首字母转大写(原文本: str) -> str:
    """
    将原文本的首字母转换为大写。

    :param 原文本: 要转换的字符串。
    :return: 首字母转为大写后的字符串。
    """
    return 原文本.capitalize()


def 文本_字符转小写(原文本: str) -> str:
    """
    将原文本中的所有字符转换为小写。可以处理 a-z 以外的字符转换。

    :param 原文本: 要转换的字符串。
    :return: 全部转为小写后的字符串。
    """
    return 原文本.lower()


def 文本_是否有汉字(原文本: str) -> bool:
    """
    检查原文本中是否含有汉字。

    :param 原文本: 要检查的字符串。
    :return: 如果原文本包含汉字，则返回 True；否则返回 False。
    """
    zhmodel = re.compile(u'[\u4e00-\u9fa5]')  # 检查中文
    return bool(zhmodel.search(原文本))


def 文本_是否全汉字(原文本: str) -> bool:
    """
    检查原文本中的字符是否全部是汉字。

    :param 原文本: 要检查的字符串。
    :return: 如果原文本全部由汉字组成，则返回 True；否则返回 False。
    """
    zhmodel = re.compile(u'[^\u4e00-\u9fa5]')  # 检查非中文字符
    return not bool(zhmodel.search(原文本))


def 文本_匹配数字(原文本: str) -> str:
    """
    返回原文本中的所有数字。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 原文本中所有数字的连续字符串。如果未匹配到数字，则返回空字符串。
    """
    结果 = re.findall(r'\d+', 原文本)
    return "".join(结果) if 结果 else ''


def 文本_匹配小写字母(原文本: str) -> str:
    """
    返回原文本中的所有小写字母。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 原文本中所有小写字母的连续字符串。如果未匹配到小写字母，则返回空字符串。
    """
    结果 = re.findall(r'[a-z]+', 原文本)
    return "".join(结果) if 结果 else ''


def 文本_匹配大写字母(原文本: str) -> str:
    """
    返回原文本中的所有大写字母。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 原文本中所有大写字母的连续字符串。如果未匹配到大写字母，则返回空字符串。
    """
    结果 = re.findall(r'[A-Z]+', 原文本)
    return "".join(结果) if 结果 else ''


def 文本_匹配字母(原文本: str) -> str:
    """
    返回原文本中的所有字母。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 原文本中所有字母的连续字符串。如果未匹配到字母，则返回空字符串。
    """
    结果 = re.findall(r'[a-zA-Z]+', 原文本)
    return "".join(结果) if 结果 else ''


def 文本_匹配手机号码(原文本: str) -> list:
    """
    匹配文本中的手机号码，以列表格式返回。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的手机号码列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"1[3-9]\d{9}", 原文本)
    return 结果 if 结果 else []


def 文本_匹配IP地址(原文本: str) -> list:
    """
    匹配文本中的IP地址，以列表格式返回。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的IP地址列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}", 原文本)
    return 结果 if 结果 else []


def 文本_匹配电话号码(原文本: str) -> list:
    """
    匹配文本中的电话号码，以列表格式返回。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的电话号码列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"\d{3,4}[\s,-]?\d{7,8}", 原文本)
    return 结果 if 结果 else []


def 文本_匹配QQ号码(原文本: str) -> list:
    """
    匹配文本中的QQ号码，以列表格式返回。最多匹配11位数QQ号。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的QQ号码列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"[1-9][0-9]{4,10}", 原文本)
    return 结果 if 结果 else []


def 文本_匹配邮政编码(原文本: str) -> list:
    """
    匹配中国邮政编码，以列表格式返回。中国邮政编码为6位数字。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的邮政编码列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"[1-9]\d{5}(?!\d)", 原文本)
    return 结果 if 结果 else []


def 文本_匹配身份证号码(原文本: str) -> list:
    """
    匹配中国的身份证号码，以列表格式返回。身份证号码为15位或18位。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的身份证号码列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"[1-9][0-9,X]{14,17}", 原文本)
    return 结果 if 结果 else []


def 文本_匹配汉字(原文本: str) -> str:
    """
    返回原文本中的所有汉字。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 原文本中所有汉字的连续字符串。如果未匹配到汉字，则返回空字符串。
    """
    结果 = re.findall(r"[\u4e00-\u9fa5]", 原文本)
    return "".join(结果)


def 文本_匹配双字节字符(原文本: str) -> str:
    """
    返回原文本中的所有双字节字符（通常包括汉字和一些全角符号）。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 原文本中所有双字节字符的连续字符串。如果未匹配到双字节字符，则返回空字符串。
    """
    结果 = re.findall(r"[^\x00-\xff]", 原文本)
    return "".join(结果)


def 文本_匹配网址(原文本: str) -> list:
    """
    匹配原文本中的网址，以列表格式返回。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的网址列表。如果未匹配到网址，则返回空列表。
    """
    结果 = re.findall(r"[a-zA-z]+://[^\s]*", 原文本)
    return 结果 if 结果 else []


def 文本_匹配IP跟端口(原文本: str) -> list:
    """
    匹配原文本中的IP地址和端口号，以列表格式返回。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的IP地址和端口号列表。如果未匹配到，则返回空列表。
    """
    结果 = re.findall(r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:\d{2,5}", 原文本)
    return 结果 if 结果 else []


def 文本_匹配邮箱号码(原文本: str) -> list:
    """
    匹配原文本中的邮箱号码，以列表格式返回。

    :param 原文本: 要进行匹配操作的字符串。
    :return: 匹配到的邮箱号码列表。如果未匹配到邮箱号码，则返回空列表。
    """
    结果 = re.findall(r"[a-z0-9\.\-+_]{1,30}@[a-z0-9\.\-+_]{1,30}\.[a-z]{1,10}", 原文本)
    return 结果 if 结果 else []


def 文本_是否正整数(原文本: str) -> bool:
    return bool(re.match(r"^[1-9]\d*$", 原文本))


def 文本_是否负整数(原文本: str) -> bool:
    return bool(re.match(r"^-[1-9]\d*$", 原文本))


def 文本_是否整数(原文本: str) -> bool:
    return bool(re.match(r"^-?[1-9]\d*$", 原文本))


def 文本_是否非正整数(原文本: str) -> bool:
    return bool(re.match(r"^-[1-9]\d*|0$", 原文本))


def 文本_是否非负整数(原文本: str) -> bool:
    return bool(re.match(r"^[1-9]\d*|0$", 原文本))


def 文本_是否正小数(原文本: str) -> bool:
    return bool(re.match(r"^[1-9]\d*\.\d*|0\.\d*[1-9]\d*$", 原文本))


def 文本_是否负小数(原文本: str) -> bool:
    return bool(re.match(r"^-([1-9]\d*\.\d*|0\.\d*[1-9]\d*)$", 原文本))


def 文本_是否小数(原文本: str) -> bool:
    return bool(re.match(r"^-?([1-9]\d*\.\d*|0\.\d*[1-9]\d*|0?\.0+|0)$", 原文本))


def 文本_是否非正小数(原文本: str) -> bool:
    return bool(re.match(r"^(-([1-9]\d*\.\d*|0\.\d*[1-9]\d*))|0?\.0+|0$", 原文本))


def 文本_是否非负小数(原文本: str) -> bool:
    return bool(re.match(r"^[1-9]\d*\.\d*|0\.\d*[1-9]\d*|0?\.0+|0$", 原文本))


def 文本_是否为IP地址(IP: str) -> bool:
    return bool(re.match(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$", IP))


def 文本_大小写翻转(原文本: str) -> str:
    """
    对原文本中的字符进行大小写翻转。

    :param 原文本: 要进行大小写翻转的字符串。
    :return: 大小写翻转后的字符串。
    """
    return 原文本.swapcase()


def 文本_是否全大写(原文本: str) -> bool:
    """
    检查原文本中的字符是否全部为大写。

    :param 原文本: 要检查的字符串。
    :return: 如果所有字符都是大写，则返回 True；否则返回 False。
    """
    return 原文本.isupper()


def 文本_是否全小写(原文本: str) -> bool:
    """
    检查原文本中的字符是否全部为小写。

    :param 原文本: 要检查的字符串。
    :return: 如果所有字符都是小写，则返回 True；否则返回 False。
    """
    return 原文本.islower()


def 文本_是否全字母(原文本: str) -> bool:
    """
    检查原文本是否全部由字母组成。

    :param 原文本: 要检查的字符串。
    :return: 如果字符串全部由字母组成，则返回 True；否则返回 False。
    """
    return 原文本.isalpha()


def 文本_是否全数字字母(原文本: str) -> bool:
    """
    检查原文本是否全部由数字和字母组成。

    :param 原文本: 要检查的字符串。
    :return: 如果字符串全部由数字和字母组成，则返回 True；否则返回 False。
    """
    return 原文本.isalnum()


def 文本_是否全数字(原文本: str) -> bool:
    """
    检查原文本是否全部由数字组成。

    :param 原文本: 要检查的字符串。
    :return: 如果字符串全部由数字组成，则返回 True；否则返回 False。
    """
    return 原文本.isdigit()


def 文本_取出现次数(原文本: str, 欲查询的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> int:
    """
    计算欲查询的文本在原文本中出现的次数。

    :param 原文本: 要进行查询的字符串。
    :param 欲查询的文本: 需要计算出现次数的字符串。
    :param 开始的位置: 查询的起始位置，默认为 0。
    :param 结束的位置: 查询的结束位置，默认为 0，表示查询到字符串的末尾。
    :return: 欲查询的文本在原文本中的出现次数。
    """
    结束的位置 = len(原文本) if 结束的位置 < 1 else 结束的位置
    return 原文本.count(欲查询的文本, 开始的位置, 结束的位置)


def 文本_是否标题化(原文本: str) -> bool:
    """
    检查原文本是否为标题化格式（所有单词的首字母大写，其余均为小写）。

    :param 原文本: 要检查的字符串。
    :return: 如果原文本是标题化的，则返回 True；否则返回 False。
    """
    return 原文本.istitle()


def 文本_标题化(原文本: str) -> str:
    """
    将原文本转换为标题化格式（所有单词的首字母大写，其余均为小写）。

    :param 原文本: 要转换的字符串。
    :return: 转换为标题化格式的字符串。
    """
    return 原文本.title()


def 文本_取文本长度(原文本: str) -> int:
    """
    返回原文本的长度。

    :param 原文本: 要测量长度的字符串。
    :return: 原文本的长度。
    """
    return len(原文本)


def 文本_是否全空格(原文本: str) -> bool:
    """
    检查原文本是否全部由空格组成。

    :param 原文本: 要检查的字符串。
    :return: 如果原文本全部由空格组成，则返回 True；否则返回 False。
    """
    return 原文本.isspace()

def 文本_删左边全部空格(原文本: str) -> str:
    """
    删除原文本左边的所有空格。

    :param 原文本: 要处理的字符串。
    :return: 删除左边空格后的字符串。
    """
    return 原文本.lstrip()


def 文本_删右边全部空格(原文本: str) -> str:
    """
    删除原文本右边的所有空格。

    :param 原文本: 要处理的字符串。
    :return: 删除右边空格后的字符串。
    """
    return 原文本.rstrip()


def 文本_删全部空格(原文本: str) -> str:
    """
    删除原文本中的所有空格。

    :param 原文本: 要处理的字符串。
    :return: 删除所有空格后的字符串。
    """
    return 原文本.replace(" ", "")


def 文本_删首尾指定字符(原文本: str, 欲删除的内容: str = ' ') -> str:
    """
    删除原文本首尾的指定字符。

    :param 原文本: 要处理的字符串。
    :param 欲删除的内容: 需要删除的字符，默认为空格。
    :return: 删除首尾指定字符后的字符串。
    """
    return 原文本.strip(欲删除的内容)


def 文本_填充空格_居中(原文本: str, 填充目标长度: int = 0) -> str:
    """
    将原文本用空格填充到指定长度，使原文本居中。

    :param 原文本: 要处理的字符串。
    :param 填充目标长度: 填充后的目标长度。
    :return: 填充后居中的字符串。
    """
    return 原文本.center(填充目标长度)


def 文本_填充空格_左对齐(原文本: str, 填充目标长度: int = 0) -> str:
    """
    将原文本用空格填充到指定长度，使原文本左对齐。

    :param 原文本: 要处理的字符串。
    :param 填充目标长度: 填充后的目标长度。
    :return: 填充后左对齐的字符串。
    """
    return 原文本.ljust(填充目标长度)


def 文本_填充空格_右对齐(原文本: str, 填充目标长度: int = 0) -> str:
    """
    将原文本用空格填充到指定长度，使原文本右对齐。

    :param 原文本: 要处理的字符串。
    :param 填充目标长度: 填充后的目标长度。
    :return: 填充后右对齐的字符串。
    """
    return 原文本.rjust(填充目标长度)


def 文本_填充0_右对齐(原文本: str, 填充目标长度: int = 0) -> str:
    """
    将原文本用 '0' 填充到指定长度，使原文本右对齐。

    :param 原文本: 要处理的字符串。
    :param 填充目标长度: 填充后的目标长度。
    :return: 填充后右对齐的字符串。
    """
    return 原文本.zfill(填充目标长度)


def 文本_拼接(连接符: str, 欲拼接的序列) -> str:
    """
    使用指定的连接符将序列中的元素拼接成一个字符串。

    :param 连接符: 用于拼接的连接符。
    :param 欲拼接的序列: 包含字符串元素的序列。
    :return: 拼接后的字符串。
    """
    return 连接符.join(欲拼接的序列)


def 文本_按键名转键值(按键名: str) -> int:
    """
    将给定的按键名转换为其对应的ASCII键值。

    :param 按键名: 要转换的按键名。
    :return: 对应的ASCII键值。
    """
    return ord(按键名)


def 文本_键值转按键名(键值: int) -> str:
    """
    将给定的ASCII键值转换为其对应的按键名。

    :param 键值: 要转换的ASCII键值。
    :return: 对应的按键名。
    """
    return chr(键值)


def 文本_是否指定文本结尾(原文本: str, 结尾的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> bool:
    """
    检查原文本是否以指定的文本结尾。

    :param 原文本: 要检查的字符串。
    :param 结尾的文本: 指定的结尾文本。
    :param 开始的位置: 检查的起始位置，默认为 0。
    :param 结束的位置: 检查的结束位置，默认为 0，表示检查到字符串的末尾。
    :return: 如果原文本以指定的文本结尾，则返回 True；否则返回 False。
    """
    结束的位置 = len(原文本) if 结束的位置 < 1 else 结束的位置
    return 原文本.endswith(结尾的文本, 开始的位置, 结束的位置)


def 文本_是否指定文本开头(原文本: str, 开头的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> bool:
    """
    检查原文本是否以指定的文本开头。

    :param 原文本: 要检查的字符串。
    :param 开头的文本: 指定的开头文本。
    :param 开始的位置: 检查的起始位置，默认为 0。
    :param 结束的位置: 检查的结束位置，默认为 0，表示检查到字符串的末尾。
    :return: 如果原文本以指定的文本开头，则返回 True；否则返回 False。
    """
    结束的位置 = len(原文本) if 结束的位置 < 1 else 结束的位置
    return 原文本.startswith(开头的文本, 开始的位置, 结束的位置)


def 文本_TAB转空格(原文本: str, 转换的数量: int = 8) -> str:
    """
    将字符串中的 tab 符号（\t）转换为指定数量的空格。

    :param 原文本: 要转换的字符串。
    :param 转换的数量: 转换为的空格数量，默认为 8。
    :return: 转换后的字符串。
    """
    return 原文本.expandtabs(tabsize=转换的数量)


def 文本_三元分割_左(原文本: str, 分割标识: str) -> tuple:
    """
    将字符串分割成三部分（分割标识前的部分，分割标识本身，分割标识后的部分）。

    :param 原文本: 要进行分割的字符串。
    :param 分割标识: 用于分割的标识。
    :return: 包含三部分的元组。
    """
    return 原文本.partition(分割标识)


def 文本_三元分割_右(原文本: str, 分割标识: str) -> tuple:
    """
    从右边开始，将字符串分割成三部分（分割标识前的部分，分割标识本身，分割标识后的部分）。

    :param 原文本: 要进行分割的字符串。
    :param 分割标识: 用于分割的标识。
    :return: 包含三部分的元组。
    """
    return 原文本.rpartition(分割标识)


def 文本_取随机邮箱() -> str:
    """
    生成随机的邮箱地址。

    :return: 随机生成的邮箱地址。
    """
    邮箱后缀 = [
        '@qq.com', '@sina.com', '@126.com', '@163.com', '@hotmail.com',
        '@139.com', '@189.com', '@sohu.com', '@21cn.com', '@189.com',
        '@tom.com', '@aol.com', '@263.com', '@aliyun.com', '@foxmail.com',
        '@yeah.net'
    ]

    def 随机字符串(长度范围, 字符集):
        return ''.join(random.choice(字符集) for _ in range(random.randint(*长度范围)))

    数字 = 随机字符串((1, 10), '123456789') + 随机字符串((0, 10), '0123456789')
    字母 = 随机字符串((5, 11), 'abcdefghijklmnopqrstuvwxyz')
    混合 = 随机字符串((1, 6), 'abcdefghijklmnopqrstuvwxyz') + 随机字符串((3, 10), '0123456789')
    字符 = 随机字符串((6, 11), '0123456789abcdefghijklmnopqrstuvwxyz')

    return random.choice([混合, 字母, 数字, 字符]) + random.choice(邮箱后缀)


def 文本_取随机手机号() -> str:
    """
    生成随机的中国大陆手机号码。

    :return: 随机生成的手机号码。
    """
    号码前缀 = [
        '130', '131', '132', '134', '135', '136', '137', '138', '139',
        '147', '150', '151', '152', '153', '155', '156', '157', '158',
        '159', '170', '171', '180', '182', '183', '185', '186', '187',
        '188', '189'
    ]
    尾号 = ''.join(str(random.randint(0, 9)) for _ in range(8))
    return random.choice(号码前缀) + 尾号


def 文本_取随机MAC地址() -> str:
    """
    生成一个随机的MAC地址。

    :return: 随机生成的MAC地址。
    """
    # MAC地址由6个16进制数表示，每个数取值范围为00到FF
    MAC片段 = [random.randint(0x00, 0xff) for _ in range(6)]
    # 将每个数转换为两位的16进制表示，然后用冒号连接
    MAC地址 = ":".join(f"{片段:02x}" for 片段 in MAC片段)
    return MAC地址


def 文本_取随机IP() -> str:
    """
    生成一个随机的IPv4地址。

    :return: 随机生成的IPv4地址。
    """
    ip = [str(random.randint(0, 255)) for _ in range(4)]
    return '.'.join(ip)


def 文本_取随机数字(取出的数量: int = 1, 是否排除0开头: bool = True) -> str:
    """
    生成指定数量的随机数字字符串。

    :param 取出的数量: 要生成的数字的数量。
    :param 是否排除0开头: 是否排除以0开头的数字。
    :return: 生成的随机数字字符串。
    """
    取出的数量 = max(1, 取出的数量)  # 确保至少生成一个数字
    文本 = str(random.randint(1, 9)) if 是否排除0开头 else str(random.randint(0, 9))
    for _ in range(取出的数量 - 1):
        文本 += str(random.randint(0, 9))
    return 文本


def 文本_取随机范围数字(最小值: int, 最大值: int, 是否返回整数: bool = False) -> int:
    """
    生成一个在指定范围内的随机数。

    :param 最小值: 随机数的最小值。
    :param 最大值: 随机数的最大值。
    :param 是否返回整数: 是否以整数形式返回。
    :return: 生成的随机数。
    """
    最小值, 最大值 = min(int(最小值), int(最大值)), max(int(最小值), int(最大值))
    return random.randint(最小值, 最大值) if 是否返回整数 else str(random.randint(最小值, 最大值))


def 文本_取随机字母(取出的数量: int = 1, 类型: int = 0) -> str:
    """
    生成指定数量的随机字母字符串。

    :param 取出的数量: 要生成的字母的数量。
    :param 类型: 字母的类型（0: 小写，1: 大写，2: 混合）。
    :return: 生成的随机字母字符串。
    """
    取出的数量 = max(1, 取出的数量)
    类型 = max(0, min(类型, 2))

    小写字母 = 'abcdefghijklmnopqrstuvwxyz'
    大写字母 = 小写字母.upper()
    字母 = 小写字母 if 类型 == 0 else 大写字母 if 类型 == 1 else 小写字母 + 大写字母

    return ''.join(random.choice(字母) for _ in range(取出的数量))


def 文本_取随机字符(取出的数量: int = 1) -> str:
    """
    生成包含数字、小写字母和大写字母的随机字符字符串。

    :param 取出的数量: 要生成的字符的数量。
    :return: 生成的随机字符字符串。
    """
    取出的数量 = max(1, 取出的数量)
    字符 = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    return ''.join(random.choice(字符) for _ in range(取出的数量))


def 文本_取随机汉字(取出的数量: int = 1) -> str:
    """
    生成指定数量的随机汉字。

    :param 取出的数量: 要生成的汉字数量。
    :return: 生成的随机汉字字符串。
    """
    取出的数量 = max(1, 取出的数量)
    return ''.join(random.choice(_常量_部分汉字) for _ in range(取出的数量))


def 文本_取随机姓氏(取常见姓氏: bool = False) -> str:
    """
    随机生成一个姓氏，可以选择是否限定为常见姓氏。

    :param 取常见姓氏: 是否只从常见姓氏中选择。
    :return: 随机生成的姓氏。
    """
    if 取常见姓氏:
        return random.choice(_常量_常见百家姓)
    else:
        return random.choice(_常量_百家姓)


_常量_百家姓 = "赵钱孙李周吴郑王冯陈褚卫蒋沈韩杨朱秦尤许何吕施张孔曹严华金魏陶姜戚谢邹喻柏水窦章云苏潘葛奚范彭郎鲁韦昌马苗凤花方俞任袁柳酆鲍史唐费廉岑薛雷贺倪汤滕殷罗毕郝邬安常乐于时傅皮卞齐康伍余元卜顾孟平黄和穆萧尹姚邵湛汪祁毛禹狄米贝明臧计伏成戴谈宋茅庞熊纪舒屈项祝董梁杜阮蓝闵席季麻强贾路娄危江童颜郭梅盛林刁钟徐邱骆高夏蔡田樊胡凌霍虞万支柯昝管卢莫柯房裘缪干解应宗丁宣贲邓郁单杭洪包诸左石崔吉钮龚程嵇邢滑裴陆荣翁荀羊于惠甄曲家封芮羿储靳汲邴糜松井段富巫乌焦巴弓牧隗山谷车侯宓蓬全郗班仰秋仲伊宫宁仇栾暴甘钭历戎祖武符刘景詹束龙叶幸司韶郜黎蓟溥印宿白怀蒲邰从鄂索咸籍赖卓蔺屠蒙池乔阳郁胥能苍双闻莘党翟谭贡劳逄姬申扶堵冉宰郦雍却璩桑桂濮牛寿通边扈燕冀浦尚农温别庄晏柴瞿阎充慕连茹习宦艾鱼容向古易慎戈廖庾终暨居衡步都耿满弘匡国文寇广禄阙东欧殳沃利蔚越夔隆师巩厍聂晁勾敖融冷訾辛阚那简饶空曾毋沙乜养鞠须丰巢关蒯相查后荆红游竺权逮盍益桓公"
_常量_常见百家姓 = "赵钱孙李周吴郑王冯陈褚卫蒋沈韩杨朱秦尤许何吕施张孔曹严华金魏陶姜戚谢邹喻柏水窦章云苏潘葛范彭郎鲁韦昌马苗凤花方俞任袁柳史唐费薛雷贺罗毕于齐萧尹姚顾孟平黄宋庞项祝董梁杜阮刘万丁石洪白田夏"
_常量_部分汉字 = _常量_百家姓 + "的一是了我不人在他有这个上们来到时大地为子中你说生国年着就那和要她出也得里后自以会家可下而过天去能对小多然于心学么之都好看起发当没成只如事把还用第样道想作种开美总从无情己面最女但现前些所同日手又行意动方期它头经长儿回位分爱老因很给名法间斯知世什两次使身者被高已亲其进此话常与活正感见明问力理尔点文几定本公特做外孩相西果走将月十实向声车全信重三机工物气每并别真打太新比才便夫再书部水像眼等体却加电主界门利海受听表德少克代员许稜先口由死安写性马光白或住难望教命花结乐色更拉东神记处让母父应直字场平报友关放至张认接告入笑内英军候民岁往何度山觉路带万男边风解叫任金快原吃妈变通师立象数四失满战远格士音轻目条呢病始达深完今提求清王化空业思切怎非找片罗钱紶吗语元喜曾离飞科言干流欢约各即指合反题必该论交终林请医晚制球决窢传画保读运及则房早院量苦火布品近坐产答星精视五连司巴奇管类未朋且婚台夜青北队久乎越观落尽形影红爸百令周吧识步希亚术留市半热送兴造谈容极随演收首根讲整式取照办强石古华諣拿计您装似足双妻尼转诉米称丽客南领节衣站黑刻统断福城故历惊脸选包紧争另建维绝树系伤示愿持千史谁准联妇纪基买志静阿诗独复痛消社算义竟确酒需单治卡幸兰念举仅钟怕共毛句息功官待究跟穿室易游程号居考突皮哪费倒价图具刚脑永歌响商礼细专黄块脚味灵改据般破引食仍存众注笔甚某沉血备习校默务土微娘须试怀料调广蜖苏显赛查密议底列富梦错座参八除跑亮假印设线温虽掉京初养香停际致阳纸李纳验助激够严证帝饭忘趣支春集丈木研班普导顿睡展跳获艺六波察群皇段急庭创区奥器谢弟店否害草排背止组州朝封睛板角况曲馆育忙质河续哥呼若推境遇雨标姐充围案伦护冷警贝著雪索剧啊船险烟依斗值帮汉慢佛肯闻唱沙局伯族低玩资屋击速顾泪洲团圣旁堂兵七露园牛哭旅街劳型烈姑陈莫鱼异抱宝权鲁简态级票怪寻杀律胜份汽右洋范床舞秘午登楼贵吸责例追较职属渐左录丝牙党继托赶章智冲叶胡吉卖坚喝肉遗救修松临藏担戏善卫药悲敢靠伊村戴词森耳差短祖云规窗散迷油旧适乡架恩投弹铁博雷府压超负勒杂醒洗采毫嘴毕九冰既状乱景席珍童顶派素脱农疑练野按犯拍征坏骨余承置臓彩灯巨琴免环姆暗换技翻束增忍餐洛塞缺忆判欧层付阵玛批岛项狗休懂武革良恶恋委拥娜妙探呀营退摇弄桌熟诺宣银势奖宫忽套康供优课鸟喊降夏困刘罪亡鞋健模败伴守挥鲜财孤枪禁恐伙杰迹妹藸遍盖副坦牌江顺秋萨菜划授归浪听凡预奶雄升碃编典袋莱含盛济蒙棋端腿招释介烧误乾坤"









# -*- coding: utf-8 -*-
import typing,sys,traceback,inspect
from functools import wraps

def 异常捕获(错误返回=None):
    def decorator(func):
        # 如果func不是函数，直接返回不装饰
        if not inspect.isfunction(func):
            return func
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception:
                # 在异常信息中包含函数名、错误详情以及调用时的参数
                # 获取传入的参数
                args_str = ", ".join(repr(a) for a in args)
                kwargs_str = ", ".join(f"{k}={v!r}" for k, v in kwargs.items())
                # 获取异常信息
                exc_type, exc_value, exc_traceback = sys.exc_info()
                # 使用 traceback 获取完整的堆栈信息
                trace_info = traceback.extract_tb(exc_traceback)
                # 最后一条记录通常是实际发生异常的位置
                last_call = trace_info[-1]
                filename = last_call.filename
                line_number = last_call.lineno
                function_name = last_call.name
                #拼接传入的参数
                if args_str and kwargs_str:
                    data = ','.join([args_str, kwargs_str])
                elif args_str:
                    data = args_str
                else:
                    data = kwargs_str

                print(f"\033[31m运行出错 {func.__name__}({data})\n错误详情 {exc_value}\n错误位置 {filename}:{line_number}\033[0m")

                if 错误返回 is not None:
                    return 错误返回
                else:
                    return_type = typing.get_type_hints(func).get('return')
                    default_values = {
                        # int: 0, #寻找文本等不适用
                        str: '',
                        bool: False,
                        float: 0.0,
                        list: [],
                        dict: {},
                        set: set(),
                        tuple: tuple(),
                        bytes: b'',
                        bytearray: bytearray(),
                        None: None
                    }
                    return default_values.get(return_type, None)
        return wrapper
    return decorator




# -*- coding:utf-8 -*-

import binascii,hashlib,base64,hmac
from urllib import parse


def 编码_UTF8编码(内容: str) -> bytes:
    """
    示例调用:
    - 编码后的内容 = 编码_UTF8编码("测试文本")
    - print(编码后的内容)  # 预期输出: b'\xe6\xb5\x8b\xe8\xaf\x95\xe6\x96\x87\xe6\x9c\xac'
    """
    return 内容.encode('UTF-8', 'strict')

def 编码_UTF8解码(内容: bytes) -> str:
    """
    示例调用:
    - 解码后的内容 = 编码_UTF8解码(b'\xe6\xb5\x8b\xe8\xaf\x95\xe6\x96\x87\xe6\x9c\xac')
    - print(解码后的内容)  # 预期输出: '测试文本'
    """
    return 内容.decode('UTF-8', 'strict')

def 编码_URL编码(内容: str, 不处理字符: str = '') -> str:
    """
    示例调用:
    - 编码后的内容 = 编码_URL编码("测试 文本")
    - print(编码后的内容)  # 预期输出: URL编码后的字符串
    """
    return parse.quote(内容, safe=不处理字符)

def 编码_URL解码(内容: str) -> str:
    """
    示例调用:
    - 解码后的内容 = 编码_URL解码("测试%20文本")
    - print(解码后的内容)  # 预期输出: '测试 文本'
    """
    return parse.unquote(内容)

def 编码_GBK编码(内容: str) -> bytes:
    """
    示例调用:
    - 编码后的内容 = 编码_GBK编码("测试文本")
    - print(编码后的内容)  # 预期输出: GBK编码的字节串
    """
    return 内容.encode('GBK', 'strict')

def 编码_GBK解码(内容: bytes) -> str:
    """
    示例调用:
    - 解码后的内容 = 编码_GBK解码(b'\xce\xca\xbd\xf0\xce\xc4\xbc\xfe')
    - print(解码后的内容)  # 预期输出: '测试文本'
    """
    return 内容.decode('GBK', 'strict')

def 编码_ANSI到USC2(内容: str) -> str:
    """
    示例调用:
    - 转换后的内容 = 编码_ANSI到USC2("测试文本")
    - print(转换后的内容)  # 预期输出: '\\u6d4b\\u8bd5\\u6587\\u672c'
    """
    return ascii(内容)

def 编码_USC2到ANSI(内容: str) -> str:
    """
    示例调用:
    - 转换后的内容 = 编码_USC2到ANSI("\\u6d4b\\u8bd5\\u6587\\u672c")
    - print(转换后的内容)  # 预期输出: '测试文本'
    """
    return 内容.encode('utf-8').decode('unicode_escape')

def 编码_BASE64编码(内容: str) -> str:
    """
    示例调用:
    - 编码后的内容 = 编码_BASE64编码("测试文本")
    - print(编码后的内容)  # 预期输出: Base64编码的字符串
    """
    if isinstance(内容, str):
        内容 = 内容.encode('UTF-8')
    return base64.b64encode(内容).decode('UTF-8')

def 编码_BASE64解码(内容: str, 返回字节集: bool = False) -> str:
    """
    示例调用:
    - 解码后的内容 = 编码_BASE64解码("编码后的Base64字符串", 返回字节集=False)
    - print(解码后的内容)  # 预期输出: 解码后的原始字符串
    """
    解码内容 = base64.b64decode(内容)
    if 返回字节集:
        return 解码内容
    else:
        return 解码内容.decode('UTF-8')

def 加密_MD5(内容: str, 编码: str = "utf-8") -> str:
    """
    计算字符串的MD5散列值。

    :param 内容: 需要计算MD5的字符串。
    :param 编码: 字符串的编码格式，默认为"utf-8"。
    :return: MD5散列值。

    示例调用:
    - md5值 = 加密_MD5("测试文本")
    - print(md5值)  # 预期输出: 字符串的MD5散列值
    """
    MD5 = hashlib.md5()
    MD5.update(内容.encode(encoding=编码))
    return MD5.hexdigest()

def 加密_SHA(内容: str, 类型: int = 2) -> str:
    """
    多种SHA加密算法。

    :param 内容: 需要加密的内容。
    :param 类型: 加密类型，0为SHA1，1为SHA224，2为SHA256，3为SHA384，4为SHA512。
    :return: 加密后的散列值。

    示例调用:
    - sha值 = 加密_SHA("测试文本", 2)
    - print(sha值)  # 预期输出: SHA256散列值
    """
    sha_functions = [hashlib.sha1, hashlib.sha224, hashlib.sha256, hashlib.sha384, hashlib.sha512]
    hash_obj = sha_functions[类型](内容.encode('utf-8'))
    return hash_obj.hexdigest()

def 加密_SHA3(内容: str, 方式: int = 0) -> str:
    """
    使用SHA3算法进行加密。

    :param 内容: 需要加密的内容。
    :param 方式: 加密方式，0为SHA3-224，1为SHA3-256，2为SHA3-384，3为SHA3-512。
    :return: 加密后的散列值。

    示例调用:
    - sha3值 = 加密_SHA3("测试文本", 1)
    - print(sha3值)  # 预期输出: SHA3-256散列值
    """
    sha3_functions = {
        0: hashlib.sha3_224(),
        1: hashlib.sha3_256(),
        2: hashlib.sha3_384(),
        3: hashlib.sha3_512()
    }
    sha3_functions[方式].update(内容.encode('utf-8'))
    return sha3_functions[方式].hexdigest()

# XOR加密作为AES加密的一个简化示例（不推荐用于实际安全需求）
def 加密_XOR(内容: str, 密钥: str) -> str:
    加密内容 = ''.join(chr(ord(c) ^ ord(密钥[i % len(密钥)])) for i, c in enumerate(内容))
    return 加密内容

def 解密_XOR(加密内容: str, 密钥: str) -> str:
    解密内容 = ''.join(chr(ord(c) ^ ord(密钥[i % len(密钥)])) for i, c in enumerate(加密内容))
    return 解密内容

def 加密_HmacSHA256(key: str, 内容: str) -> str:
    """
    使用HMAC和SHA256算法进行加密。

    :param key: 加密密钥。
    :param 内容: 需要加密的内容。
    :return: 加密后的字符串（Base64编码）。

    示例调用:
    - 加密结果 = 加密_HmacSHA256("密钥", "测试文本")
    - print(加密结果)  # 预期输出: HMAC SHA256加密后的Base64字符串
    """
    return base64.b64encode(hmac.new(key.encode('utf-8'), 内容.encode('utf-8'), digestmod=hashlib.sha256).digest()).decode("utf-8")

def 加密_CRC32(内容: str) -> int:
    """
    使用CRC32算法进行加密。

    :param 内容: 需要加密的内容。
    :return: 加密后的整数值。

    示例调用:
    - crc32值 = 加密_CRC32("测试文本")
    - print(crc32值)  # 预期输出: CRC32加密后的整数值
    """
    return binascii.crc32(内容.encode("utf-8"))

def 进制_二到八(内容: str) -> str:
    """
    示例调用:
    - 二进制内容 = '1010'
    - 八进制内容 = 进制_二到八(二进制内容)
    - print(八进制内容)  # 预期输出: '0o12'
    """
    return oct(int(内容, 2))

def 进制_二到十(内容: str) -> int:
    """
    示例调用:
    - 二进制内容 = '1010'
    - 十进制内容 = 进制_二到十(二进制内容)
    - print(十进制内容)  # 预期输出: 10
    """
    return int(内容, 2)

def 进制_二到十六(内容: str) -> str:
    """
    示例调用:
    - 二进制内容 = '1010'
    - 十六进制内容 = 进制_二到十六(二进制内容)
    - print(十六进制内容)  # 预期输出: '0xa'
    """
    return hex(int(内容, 2))

def 进制_八到二(内容: str) -> str:
    """
    示例调用:
    - 八进制内容 = '12'
    - 二进制内容 = 进制_八到二(八进制内容)
    - print(二进制内容)  # 预期输出: '0b1010'
    """
    return bin(int(内容, 8))

def 进制_八到十(内容: str) -> int:
    """
    示例调用:
    - 八进制内容 = '12'
    - 十进制内容 = 进制_八到十(八进制内容)
    - print(十进制内容)  # 预期输出: 10
    """
    return int(内容, 8)

def 进制_八到十六(内容: str) -> str:
    """
    示例调用:
    - 八进制内容 = '12'
    - 十六进制内容 = 进制_八到十六(八进制内容)
    - print(十六进制内容)  # 预期输出: '0xa'
    """
    return hex(int(内容, 8))

def 进制_十到二(内容: int) -> str:
    """
    示例调用:
    - 十进制内容 = 10
    - 二进制内容 = 进制_十到二(十进制内容)
    - print(二进制内容)  # 预期输出: '0b1010'
    """
    return bin(内容)

def 进制_十到八(内容: int) -> str:
    """
    示例调用:
    - 十进制内容 = 10
    - 八进制内容 = 进制_十到八(十进制内容)
    - print(八进制内容)  # 预期输出: '0o12'
    """
    return oct(内容)

def 进制_十到十六(内容: int) -> str:
    """
    示例调用:
    - 十进制内容 = 10
    - 十六进制内容 = 进制_十到十六(十进制内容)
    - print(十六进制内容)  # 预期输出: '0xa'
    """
    return hex(内容)

def 进制_十六到二(内容: str) -> str:
    """
    示例调用:
    - 十六进制内容 = 'a'
    - 二进制内容 = 进制_十六到二(十六进制内容)
    - print(二进制内容)  # 预期输出: '0b1010'
    """
    return bin(int(内容, 16))

def 进制_十六到八(内容: str) -> str:
    """
    示例调用:
    - 十六进制内容 = 'a'
    - 八进制内容 = 进制_十六到八(十六进制内容)
    - print(八进制内容)  # 预期输出: '0o12'
    """
    return oct(int(内容, 16))

def 进制_十六到十(内容: str) -> int:
    """
    示例调用:
    - 十六进制内容 = 'a'
    - 十进制内容 = 进制_十六到十(十六进制内容)
    - print(十进制内容)  # 预期输出: 10
    """
    return int(内容, 16)


# -*- coding: utf-8 -*-
import socket,subprocess,winreg,ctypes,os,sys,re,uuid,signal


def 系统_取本机电脑名称() -> str:
    """
    获取本机电脑的名称。

    :return: 返回电脑的名称。如果发生异常，则返回空。
    """
    return socket.gethostname()


def 系统_执行Dos(命令: str) -> bool:
    """
    使用subprocess执行给定的DOS命令。

    :param 命令: 要执行的命令行命令。
    :return: 命令执行成功返回True，失败返回False。
    """
    try:
        result = subprocess.run(命令, shell=True, check=True)
        return True
    except subprocess.CalledProcessError:
        return False


def 系统_执行Dos_带返回(命令: str) -> str:
    """
    使用subprocess执行给定的DOS命令，并返回命令的输出结果。

    :param 命令: 要执行的命令行命令。
    :return: 返回命令的标准输出结果。
    """
    with subprocess.Popen(命令, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE) as p:
        stdout, stderr = p.communicate()
        if p.returncode == 0:
            return stdout.decode('utf-8')
        else:
            return stderr.decode('utf-8')


def 系统_执行Dos_os(命令: str) -> bool:
    """
    使用os.system执行给定的DOS命令。

    :param 命令: 要执行的命令行命令。
    :return: 命令执行成功返回True，失败返回False。
    """
    状态码 = os.system(命令)
    return 状态码 == 0


def 系统_执行Dos_带返回_os(命令: str) -> str:
    """
    使用os.popen执行给定的DOS命令，并返回命令的输出结果。

    :param 命令: 要执行的命令行命令。
    :return: 返回命令的标准输出结果。
    """
    with os.popen(命令) as f:
        结果 = f.read()
    return 结果


def 系统_连接宽带(宽带名称: str, 用户名: str, 密码: str) -> bool:
    """
    尝试使用指定的用户名和密码连接到宽带。

    :param 宽带名称: 要连接的宽带的名称。
    :param 用户名: 宽带账户的用户名。
    :param 密码: 宽带账户的密码。
    :return: 一个元组，第一个元素是一个布尔值，表示连接是否成功，第二个元素是连接结果的描述。
    """
    result = subprocess.run(["rasdial", 宽带名称, 用户名, 密码], capture_output=True, text=True, check=True)
    return True


def 系统_断开宽带(宽带名称: str) -> bool:
    """
    尝试断开指定宽带的连接。

    :param 宽带名称: 要断开的宽带的名称。
    :return: 一个元组，第一个元素是一个布尔值，表示断开是否成功，第二个元素是断开结果的描述。
    """
    result = subprocess.run(["rasdial", 宽带名称, "/DISCONNECT"], capture_output=True, text=True, check=True)
    return True


def 系统_设置系统代理(代理状态: bool, 代理地址: str = '') -> bool:
    """
    设置或取消Windows系统代理。

    :param 代理状态: 一个布尔值，指示是否启用代理。
    :param 代理地址: 代理服务器的地址和端口，格式如 '127.0.0.1:8080'。
    :return: 成功返回True,失败返回False
    """
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                         r'Software\Microsoft\Windows\CurrentVersion\Internet Settings',
                         0, winreg.KEY_ALL_ACCESS)

    def 设置键值(名称, 值):
        _, reg_type = winreg.QueryValueEx(key, 名称)
        winreg.SetValueEx(key, 名称, 0, reg_type, 值)

    设置键值('ProxyEnable', int(代理状态))
    if 代理状态 and 代理地址:
        设置键值('ProxyServer', 代理地址)

    winreg.CloseKey(key)
    return True


def 系统_获取代理状态() -> bool:
    """
    获取Windows系统代理是否启用的状态。

    :return: 如果代理启用，返回True；否则返回False。
    """
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                         r"Software\Microsoft\Windows\CurrentVersion\Internet Settings",
                         0, winreg.KEY_READ)
    try:
        代理状态, _ = winreg.QueryValueEx(key, "ProxyEnable")
        return bool(代理状态)
    finally:
        winreg.CloseKey(key)


def 系统_获取代理地址() -> str:
    """
    获取Windows系统代理服务器的地址。

    :return: 返回代理服务器的地址字符串。如果代理未启用，返回空字符串。
    """
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                         r"Software\Microsoft\Windows\CurrentVersion\Internet Settings",
                         0, winreg.KEY_READ)
    try:
        代理状态, _ = winreg.QueryValueEx(key, "ProxyEnable")
        if bool(代理状态):
            代理服务器, _ = winreg.QueryValueEx(key, "ProxyServer")
            return 代理服务器
        else:
            return ""
    finally:
        winreg.CloseKey(key)


def 系统_是否管理员运行() -> bool:
    """
    检查当前程序是否以管理员权限运行。

    :return: 如果是管理员权限则返回True，否则返回False。
    """
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except:
        return False

def 系统_请求管理员权限() -> bool:
    """
    如果当前程序不是以管理员权限运行，则请求提升权限。
    """
    if not 系统_是否管理员运行():
        ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, " ".join(sys.argv), None, 1)
    return True

def 系统_取Python安装目录() -> str:
    """
    获取当前运行的Python解释器的安装目录。

    :return: Python解释器的完整路径。
    """
    return sys.executable

def 系统_取网络接口名() -> list:
    """
    获取当前活动的网络接口名称。

    :return: 包含活动网络接口名称的列表。
    """
    try:
        # 执行ipconfig命令并获取输出
        结果 = subprocess.run(["ipconfig"], capture_output=True, text=True)
        输出 = 结果.stdout

        # 使用正则表达式查找接口名称
        接口名列表 = re.findall(r"适配器 (.+?):", 输出)
        return 接口名列表
    except Exception as e:
        print(f"获取活动网络接口名时出错: {e}")
        return []

def 系统_修改DNS(接口名: str, 主DNS: str, 备用DNS: str = '') -> bool:
    """
    修改指定网络接口的主DNS和备用DNS地址。

    :param 接口名: 要修改的网络接口名称,可通过系统_取网络接口名()获取。
    :param 主DNS: 新的主DNS地址。
    :param 备用DNS: 新的备用DNS地址（可选）。
    :return: 操作成功返回True，否则返回False。

    示例调用：
    - 系统_修改DNS("以太网", "8.8.8.8", "8.8.4.4")
    """
    try:
        网络接口键路径 = f"SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces\\{接口名}"
        with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, 网络接口键路径, 0, winreg.KEY_WRITE) as 键:
            DNS设置 = 主DNS
            if 备用DNS:
                DNS设置 += " " + 备用DNS
            winreg.SetValueEx(键, "NameServer", 0, winreg.REG_SZ, DNS设置)

        # 重新应用网络设置
        subprocess.run(["ipconfig", "/flushdns"], check=True)
        subprocess.run(["ipconfig", "/registerdns"], check=True)
        subprocess.run(["ipconfig", "/release"], check=True)
        subprocess.run(["ipconfig", "/renew"], check=True)
        return True
    except Exception as e:
        print(f"修改DNS时出错: {e}")
        return False

def 系统_修改MAC地址(接口名: str, 新MAC地址: str) -> bool:
    """
    修改指定网络接口的MAC地址。

    :param 接口名: 网络接口的名称，例如 "以太网,可通过系统_取网络接口名()获取"。
    :param 新MAC地址: 要设置的新MAC地址。
    :return: 操作成功返回True，否则返回False。

    示例调用：
    - 系统_修改MAC地址("以太网", "00-11-22-33-44-55")
    """
    try:
        网络接口键路径 = f"SYSTEM\\CurrentControlSet\\Control\\Class\\{{4d36e972-e325-11ce-bfc1-08002be10318}}"
        接口键 = None

        # 打开网络适配器注册表
        with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, 网络接口键路径) as 键:
            for i in range(1000):
                try:
                    接口键名 = winreg.EnumKey(键, i)
                    接口键路径 = f"{网络接口键路径}\\{接口键名}"
                    with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, 接口键路径) as 接口键:
                        接口值, _ = winreg.QueryValueEx(接口键, "NetCfgInstanceId")
                        if 接口值 == 接口名:
                            break
                except WindowsError:
                    break

        if 接口键 is None:
            raise ValueError("未找到指定的网络接口")

        # 设置新的MAC地址
        with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, 接口键路径, 0, winreg.KEY_WRITE) as 接口键:
            winreg.SetValueEx(接口键, "NetworkAddress", 0, winreg.REG_SZ, 新MAC地址)

        # 重启网络接口以应用更改
        subprocess.run(["netsh", "interface", "set", "interface", 接口名, "disable"], check=True)
        subprocess.run(["netsh", "interface", "set", "interface", 接口名, "enable"], check=True)
        return True
    except Exception as e:
        print(f"修改MAC地址时出错: {e}")
        return False

def 系统_修改主页(新主页: str) -> bool:
    """
    修改Internet Explorer的主页。

    :param 新主页: 要设置的新主页URL。
    :return: 操作成功返回True，否则返回False。

    示例调用：
    - 系统_修改主页("https://www.example.com")
    """
    try:
        键路径 = r"Software\Microsoft\Internet Explorer\Main"
        with winreg.OpenKey(winreg.HKEY_CURRENT_USER, 键路径, 0, winreg.KEY_WRITE) as 键:
            winreg.SetValueEx(键, "Start Page", 0, winreg.REG_SZ, 新主页)
        return True
    except Exception as e:
        print(f"修改主页时出错: {e}")
        return False

def 系统_关机():
    """
    立即关机计算机。
    """
    subprocess.run(["shutdown", "/s", "/f", "/t", "0"], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def 系统_注销():
    """
    注销当前用户。
    """
    subprocess.run(["shutdown", "/l"], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def 系统_重启():
    """
    重启计算机。
    """
    subprocess.run(["shutdown", "/r", "/f", "/t", "0"], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def 系统_重启资源管理器():
    """
    重启Windows资源管理器。
    """
    # 结束资源管理器进程
    subprocess.run(["taskkill", "/f", "/im", "explorer.exe"], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    # 重新启动资源管理器
    subprocess.Popen("explorer.exe", stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def 系统_切换用户():
    """
    锁定当前会话，以便用户可以切换到其他账户。
    """
    ctypes.windll.user32.LockWorkStation()

def 系统_取数值注册项(路径: str, 名称: str) -> int:
    """
    从注册表中读取数值型注册项。

    :param 路径: 注册表项的路径。
    :param 名称: 注册表项的名称。
    :return: 注册表项的数值。

    示例调用：
    - print(系统_取数值注册项("HKEY_CURRENT_USER\\Software\\MyApp", "Setting"))
    """
    try:
        根键名, 子键路径 = 路径.split("\\", 1)
        根键 = getattr(winreg, 根键名)
        with winreg.OpenKey(根键, 子键路径) as 键:
            值, 类型 = winreg.QueryValueEx(键, 名称)
            return 值
    except Exception as e:
        print(f"读取数值注册项时出错: {e}")
        return 0

def 系统_取文本注册项(路径: str, 名称: str) -> str:
    """
    从注册表中读取文本型注册项。

    :param 路径: 注册表项的路径。
    :param 名称: 注册表项的名称。
    :return: 注册表项的文本值。

    示例调用：
    - print(系统_取文本注册项("HKEY_CURRENT_USER\\Software\\MyApp", "Setting"))
    """
    try:
        根键名, 子键路径 = 路径.split("\\", 1)
        根键 = getattr(winreg, 根键名)
        with winreg.OpenKey(根键, 子键路径) as 键:
            值, 类型 = winreg.QueryValueEx(键, 名称)
            return 值
    except Exception as e:
        print(f"读取文本注册项时出错: {e}")
        return ""

def 系统_写数值注册项(路径: str, 名称: str, 值: int, 类型: str = "DWORD"):
    """
    写入数值型的注册表项。

    :param 路径: 注册表项的路径。
    :param 名称: 注册表项的名称。
    :param 值: 要写入的数值。
    :param 类型: 注册表项的类型（"DWORD" 或 "QWORD"）。
    :return: 操作成功返回True，否则返回False。

    示例调用：
    - 系统_写数值注册项("HKEY_CURRENT_USER\\Software\\MyApp", "Setting", 1)
    """
    try:
        # 确定根键和子键
        根键名, 子键路径 = 路径.split("\\", 1)
        根键 = getattr(winreg, 根键名)

        # 打开或创建键
        with winreg.CreateKey(根键, 子键路径) as 键:
            if 类型 == "DWORD":
                winreg.SetValueEx(键, 名称, 0, winreg.REG_DWORD, 值)
            elif 类型 == "QWORD":
                winreg.SetValueEx(键, 名称, 0, winreg.REG_QWORD, 值)
        return True
    except Exception as e:
        print(f"写入注册表时出错: {e}")
        return False

def 系统_写文本注册项(路径: str, 名称: str, 值: str):
    """
    写入文本型的注册表项。

    :param 路径: 注册表项的路径。
    :param 名称: 注册表项的名称。
    :param 值: 要写入的文本值。
    :return: 操作成功返回True，否则返回False。

    示例调用：
    - 系统_写文本注册项("HKEY_CURRENT_USER\\Software\\MyApp", "Setting", "value")
    """
    try:
        # 确定根键和子键
        根键名, 子键路径 = 路径.split("\\", 1)
        根键 = getattr(winreg, 根键名)

        # 打开或创建键
        with winreg.CreateKey(根键, 子键路径) as 键:
            winreg.SetValueEx(键, 名称, 0, winreg.REG_SZ, 值)
        return True
    except Exception as e:
        print(f"写入注册表时出错: {e}")
        return False

def 系统_删除注册项(路径: str, 名称: str) -> bool:
    """
    删除指定的注册表项。

    :param 路径: 注册表项的路径。
    :param 名称: 注册表项的名称。
    :return: 操作成功返回True，否则返回False。

    示例调用：
    - 系统_删除注册项("HKEY_CURRENT_USER\\Software\\MyApp", "Setting")
    """
    try:
        根键名, 子键路径 = 路径.split("\\", 1)
        根键 = getattr(winreg, 根键名)
        with winreg.OpenKey(根键, 子键路径, 0, winreg.KEY_WRITE) as 键:
            winreg.DeleteValue(键, 名称)
        return True
    except Exception as e:
        print(f"删除注册表项时出错: {e}")
        return False

def 系统_注册项是否存在(路径: str, 名称: str) -> bool:
    """
    检查指定的注册表项是否存在。

    :param 路径: 注册表项的路径。
    :param 名称: 注册表项的名称。
    :return: 存在返回True，否则返回False。

    示例调用：
    - print(系统_注册项是否存在("HKEY_CURRENT_USER\\Software\\MyApp", "Setting"))
    """
    try:
        根键名, 子键路径 = 路径.split("\\", 1)
        根键 = getattr(winreg, 根键名)
        with winreg.OpenKey(根键, 子键路径) as 键:
            winreg.QueryValueEx(键, 名称)
        return True
    except FileNotFoundError:
        return False

def 系统_是否有摄像头() -> bool:
    """
    检查系统是否有摄像头。

    :return: 如果有摄像头返回True，否则返回False。
    """
    结果 = subprocess.run(["wmic", "path", "Win32_PnPEntity", "where", "DeviceID like 'USB%'", "get", "Name"], capture_output=True, text=True)
    return "摄像头" in 结果.stdout

def 系统_打开任务管理器() -> bool:
    """
    打开Windows任务管理器,需要权限。
    """
    try:
        subprocess.Popen("taskmgr.exe")
        return True
    except Exception as e:
        print(f"打开任务管理器时出错: {e}")
        return False

def 系统_打开控制面板() -> bool:
    """
    打开Windows控制面板。
    """
    try:
        subprocess.Popen(["control"])
        return True
    except Exception as e:
        print(f"打开控制面板时出错: {e}")
        return False

def 系统_打开网络连接属性():
    """
    打开网络连接属性窗口。
    """
    try:
        subprocess.Popen(["control", "ncpa.cpl"])
        return True
    except Exception as e:
        print(f"打开网络连接属性时出错: {e}")
        return False

def 系统_恢复屏保() -> bool:
    """
    恢复屏幕保护程序的默认设置。

    :return: 操作结果字符串。
    """
    try:
        with winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Control Panel\Desktop", 0, winreg.KEY_WRITE) as 键:
            屏保激活, _ = winreg.QueryValueEx(键, "ScreenSaveActive")
            if 屏保激活 != "1":
                winreg.SetValueEx(键, "ScreenSaveActive", 0, winreg.REG_SZ, "1")
            return True
    except Exception as e:
        return False

def 系统_去掉屏保():
    """
    禁用Windows的屏幕保护程序。

    示例调用：
    - 系统_去掉屏保()
    """
    try:
        with winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Control Panel\Desktop", 0, winreg.KEY_WRITE) as 键:
            winreg.SetValueEx(键, "ScreenSaveActive", 0, winreg.REG_SZ, "0")
        return True
    except Exception as e:
        print(f"去掉屏保时出错: {e}")
        return False


def 系统_取MAC地址() -> str:
    """
    获取计算机的MAC地址。

    :return: MAC地址字符串。
    """
    mac = uuid.getnode()
    mac_str = ':'.join(('%012X' % mac)[i:i+2] for i in range(0, 12, 2))
    return mac_str


def 系统_取主页地址() -> str:
    """
    获取Internet Explorer的主页地址。

    :return: 主页地址字符串。
    """
    try:
        with winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Software\Microsoft\Internet Explorer\Main") as 键:
            主页地址, _ = winreg.QueryValueEx(键, "Start Page")
            return 主页地址
    except Exception as e:
        return ""

def 系统_取屏幕分辨率() -> tuple:
    """
    获取当前屏幕的分辨率。

    :return: 屏幕分辨率，格式为 (宽度, 高度)。
    """
    user32 = ctypes.windll.user32
    屏幕宽度 = user32.GetSystemMetrics(0)
    屏幕高度 = user32.GetSystemMetrics(1)
    return (屏幕宽度, 屏幕高度)

def 系统_取屏幕数量() -> int:
    """
    获取系统连接的显示器数量。

    :return: 连接的显示器数量。
    """
    user32 = ctypes.windll.user32
    屏幕数量 = user32.GetSystemMetrics(80)  # SM_CMONITORS
    return 屏幕数量

def 系统_取内网IP列表() -> list:
    """
    获取当前设备的内网IP地址。

    :return: 内网IP地址字符串的列表。
    """
    try:
        # 执行ipconfig命令并获取输出
        结果 = subprocess.run(["ipconfig"], capture_output=True, text=True)
        输出 = 结果.stdout
        # 使用正则表达式查找IPv4地址
        匹配 = re.findall(r'IPv4 地址[ .:]*([0-9]+(?:\.[0-9]+){3})', 输出)
        if 匹配:
            return 匹配
    except Exception as e:
        pass
    return []

def 系统_取系统DPI() -> int:
    """
    获取当前系统的DPI设置。

    :return: 系统DPI值。
    """
    # 创建一个设备上下文（DC）对象用于屏幕
    user32 = ctypes.windll.user32
    hdc = user32.GetDC(0)
    # 获取水平DPI值
    DPI = ctypes.windll.gdi32.GetDeviceCaps(hdc, 88)  # 88是水平DPI的索引
    user32.ReleaseDC(0, hdc)
    return DPI

def 系统_取系统信息() -> str:
    """
    获取并返回系统的详细信息。

    :return: 系统信息的字符串。
    """
    try:
        系统信息 = subprocess.check_output("systeminfo", encoding="gbk")
        return 系统信息
    except Exception as e:
        return ""

def 系统_桌面图标显示() -> bool:
    """
    显示桌面图标。
    """
    try:
        SPI_SETICONS = 0x0059
        SPIF_UPDATEINIFILE = 0x01
        SPIF_SENDCHANGE = 0x02
        ctypes.windll.user32.SystemParametersInfoW(SPI_SETICONS, 1, None, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE)
        return True
    except Exception as e:
        print(f"显示桌面图标时出错: {e}")
        return False

def 系统_桌面图标隐藏() -> bool:
    """
    隐藏桌面图标。
    """
    try:
        SPI_SETICONS = 0x0059
        SPIF_UPDATEINIFILE = 0x01
        SPIF_SENDCHANGE = 0x02
        ctypes.windll.user32.SystemParametersInfoW(SPI_SETICONS, 0, None, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE)
        return True
    except Exception as e:
        print(f"隐藏桌面图标时出错: {e}")
        return False

def 系统_清空回收站() -> bool:
    """
    清空Windows回收站。
    """
    try:
        ctypes.windll.shell32.SHEmptyRecycleBinW(None, None, 0)
        return True
    except Exception as e:
        print(f"清空回收站时出错: {e}")
        return False

def 系统_显示我的电脑() -> bool:
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Explorer\HideDesktopIcons\NewStartPanel', 0, winreg.KEY_SET_VALUE)
    winreg.SetValueEx(key, '{20D04FE0-3AEA-1069-A2D8-08002B30309D}', 0, winreg.REG_DWORD, 0)
    winreg.CloseKey(key)
    return True

def 系统_隐藏我的电脑() -> bool:
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Explorer\HideDesktopIcons\NewStartPanel', 0, winreg.KEY_SET_VALUE)
    winreg.SetValueEx(key, '{20D04FE0-3AEA-1069-A2D8-08002B30309D}', 0, winreg.REG_DWORD, 1)
    winreg.CloseKey(key)
    return True

def 系统_隐藏已知文件扩展名() -> bool:
    key_path = r'Software\Microsoft\Windows\CurrentVersion\Explorer\Advanced'
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, key_path, 0, winreg.KEY_SET_VALUE)
    winreg.SetValueEx(key, 'HideFileExt', 0, winreg.REG_DWORD, 1)
    winreg.CloseKey(key)
    return True

def 系统_显示已知文件扩展名() -> bool:
    key_path = r'Software\Microsoft\Windows\CurrentVersion\Explorer\Advanced'
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, key_path, 0, winreg.KEY_SET_VALUE)
    winreg.SetValueEx(key, 'HideFileExt', 0, winreg.REG_DWORD, 0)
    winreg.CloseKey(key)
    return True

def 系统_关闭系统更新() -> bool:
    key_path = r'SOFTWARE\Policies\Microsoft\Windows\WindowsUpdate\AU'
    key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, key_path, 0, winreg.KEY_WRITE)
    winreg.SetValueEx(key, 'NoAutoUpdate', 0, winreg.REG_DWORD, 1)
    winreg.CloseKey(key)
    return True

def 程序_立即结束():
    """
    立即结束当前运行的Python程序。
    """
    sys.exit()


def 进程_强制结束():
    '使用kill结束当前进程'
    try:
        os.kill(os.getpid(), signal.SIGTERM)  # 发送SIGTERM信号尝试正常结束进程
    except OSError as e:
        print(f"结束进程失败: {e}")


def 进程_结束(进程ID: int):
    """
    结束指定ID的进程。

    :param 进程ID: 要结束的进程的ID。
    """
    try:
        os.kill(进程ID, signal.SIGTERM)  # 发送SIGTERM信号尝试正常结束进程
    except OSError as e:
        print(f"结束进程失败: {e}")


def 进程_取当前进程ID() -> int:
    """
    获取当前运行程序的进程ID。

    :return: 返回当前进程的ID。
    """
    return os.getpid()


# -*- coding: utf-8 -*-
import time,execjs,re,base64,struct,hmac,hashlib
from io import BytesIO, StringIO
from typing import Any
import os,sys



def 程序_是否调试运行() -> bool:
    """
    检查程序是否从源码运行,如果打包后运行则返回False。
    """
    # 当使用 PyInstaller 打包时，它会创建一个临时文件夹，该文件夹的路径被存储在 _MEIPASS 中
    return not getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS')

class __IO共用类:

    def 读取(self, 长度: int = -1) -> Any:
        """
        读取指定长度的内容。长度为-1时读取全部内容。

        :param 长度: 要读取的内容长度，-1表示读取全部内容。
        :return: 读取到的内容字符串。
        """
        return self.read(长度)

    def 读取全部(self) -> str:
        """
        读取流中的全部内容。

        :return: 流中的全部内容字符串。
        """
        return self.getvalue()

    def 写入(self, 内容: Any):
        """
        写入内容到流中。

        :param 内容: 要写入的内容字符串。
        """
        self.write(内容)

    def 写入多行(self, 行列表: list):
        """
        写入多行内容到流中。

        :param 行列表: 包含多行内容的列表。
        """
        self.writelines(行列表)

    def 关闭(self):
        """
        关闭流。
        """
        self.close()

    def 刷新(self):
        """
        刷新流的缓冲区。
        """
        self.flush()

    def 移动到(self, 位置: int):
        """
        移动流的读写位置到指定位置。

        :param 位置: 流中的目标位置。
        """
        self.seek(位置)

    def 当前位置(self) -> int:
        """
        返回当前流的读写位置。

        :return: 当前流的位置。
        """
        return self.tell()

    def 调整大小(self, 新大小: int) -> int:
        """
        调整流的大小。

        :param 新大小: 流的新大小。
        :return: 调整后的流的大小。
        """
        return self.truncate(新大小)

    def 读取行(self) -> str:
        """
        读取流中的一行内容。

        :return: 流中的一行内容字符串。
        """
        return self.readline()

    def 读取所有行(self) -> list:
        """
        读取流中的所有行，返回列表。

        :return: 包含流中所有行的列表。
        """
        return self.readlines()

    def 是否关闭(self) -> bool:
        """
        检查流是否已关闭。

        :return: 如果流已关闭则返回True，否则返回False。
        """
        return self.closed

    def 是否可读(self) -> bool:
        """
        检查流是否可读。

        :return: 如果流可读则返回True，否则返回False。
        """
        return self.readable()

    def 是否可写(self) -> bool:
        """
        检查流是否可写。

        :return: 如果流可写则返回True，否则返回False。
        """
        return self.writable()

    def 是否支持随机访问(self) -> bool:
        """
        检查流是否支持随机访问。

        :return: 如果流支持随机访问则返回True，否则返回False。
        """
        return self.seekable()


class IO_二进制(__IO共用类, BytesIO):
    """BytesIO 处理二进制数据的流。"""

    def 写入(self, 内容: bytes):
        """写入二进制内容到流中。"""
        super().写入(内容)

    def 读取(self, 长度: int = -1) -> bytes:
        """读取指定长度的二进制内容。"""
        return super().读取(长度)

class IO_文本(__IO共用类, StringIO):
    """StringIO 处理文本数据的流。"""

    def 写入(self, 内容: str):
        """写入文本内容到流中。"""
        super().写入(内容)

    def 读取(self, 长度: int = -1) -> str:
        """读取指定长度的文本内容。"""
        return super().读取(长度)


class JavaScript执行器:
    def __init__(self, JS代码: str):
        """
        初始化JavaScript执行器。

        :param JS代码: 要执行的JavaScript代码。
        """
        self.js环境 = execjs.compile(JS代码)

    def 运行(self, 方法名: str, *参数) -> any:
        """
        执行JavaScript代码中的指定方法。

        :param 方法名: JavaScript方法的名称。
        :param 参数: 传递给JavaScript方法的参数。
        :return: JavaScript方法的返回值。
        """
        return self.js环境.call(方法名, *参数)


def 调试输出(*args, 颜色=None):
    """
    打印调试输出，可以选择不同的颜色。

    :param args: 要打印的内容。
    :param 颜色: 打印内容的颜色。可选值包括 '红色', '绿色', '黄色', '蓝色', '紫色', '青色', '白色'。
    """
    颜色代码 = {
        '红色': '\033[91m',
        '绿色': '\033[92m',
        '黄色': '\033[93m',
        '蓝色': '\033[94m',
        '紫色': '\033[95m',
        '青色': '\033[96m',
        '白色': '\033[97m'
    }

    颜色前缀 = 颜色代码.get(颜色, '')
    颜色后缀 = '\033[0m' if 颜色 else ''  # 如果指定了颜色，则在末尾重置

    print(颜色前缀, *args, 颜色后缀)


def 程序_延时(秒数: float):
    """
    实现程序的延时。

    :param 秒数: 要延迟的时间，单位为秒。可以是小数。
    """
    time.sleep(秒数)


def 正则_匹配(原文本: str, 匹配规则: str) -> list:
    """
    根据给定的正则表达式匹配规则，在原文本中查找所有匹配项。

    :param 原文本: 要进行匹配的文本。
    :param 匹配规则: 正则表达式匹配规则。
    :return: 匹配到的所有结果的列表。
    """
    return re.findall(匹配规则, 原文本)


def 谷歌身份验证(secret_key: str) -> str:
    """
    基于时间的谷歌身份验证算法，生成动态验证码。

    :param secret_key: 身份验证器中使用的密钥。
    :return: 生成的六位数动态验证码。
    """
    for x in range(5):  # 最多尝试5次补全
        try:
            decode_secret = base64.b32decode(secret_key, True)
            interval_number = int(time.time() // 30)
            message = struct.pack(">Q", interval_number)
            digest = hmac.new(decode_secret, message, hashlib.sha1).digest()
            index = ord(digest[19:20]) % 16
            google_code = (struct.unpack(">I", digest[index:index + 4])[0] & 0x7fffffff) % 1000000
            return "%06d" % google_code
        except Exception as e:
            secret_key += "="  # 尝试补全密钥

    raise ValueError("无法生成有效的谷歌验证码。请检查密钥是否正确。")


def 运行python代码(代码: str, 全局变量: dict = None, 局部变量: dict = None) -> any:
    """
    动态执行python代码字符串并返回表达式的结果。

    :param 代码: 要执行的python代码字符串。
    :param 全局变量: 全局变量字典。
    :param 局部变量: 局部变量字典。
    :return: 执行代码的结果。
    """
    return eval(代码, 全局变量, 局部变量)


def 运行python代码2(代码: str, 全局变量: dict = None, 局部变量: dict = None) -> None:
    """
    动态执行python代码，不返回结果。

    :param 代码: 要执行的python代码字符串。
    :param 全局变量: 全局变量字典。
    :param 局部变量: 局部变量字典。
    """
    exec(代码, 全局变量, 局部变量)


# -*- coding: utf-8 -*-
import pkgutil,types,importlib
from .装饰器 import *


def 自动装饰器_应用(包名):
    包 = importlib.import_module(包名)
    for 加载器, 模块名, 是包 in pkgutil.walk_packages(包.__path__, 包.__name__ + '.'):
        if not 是包:
            模块 = importlib.import_module(模块名)
            for 属性名 in dir(模块):
                属性 = getattr(模块, 属性名)
                if isinstance(属性, types.FunctionType):
                    装饰后的函数 = 异常捕获()(属性)
                    setattr(模块, 属性名, 装饰后的函数)

自动装饰器_应用('Taowa')



from .数据类型类 import *
from .网页 import *
from .编码 import *
from .文本 import *
from .文件 import *
from .系统 import *
from .时间 import *
from .线程 import *
from .其它 import *




# -*- coding: utf-8 -*-
import sys,os,stat,hashlib
from functools import partial


def 文件_取运行目录() -> str:
    """
    获取当前工作目录的路径。

    :return: 当前工作目录的路径字符串。
    """
    return os.getcwd()

def 文件_取临时目录(目录: str = '.') -> str:
    """
    获取临时目录的路径。在打包的应用程序中用于获取资源文件的地址。

    :param 目录: 默认当前目录，可指定其他目录。
    :return: 临时目录的路径字符串。
    """
    return sys._MEIPASS if getattr(sys, "frozen", False) else os.path.abspath(目录)

def 文件_遍历指定路径文件(路径: str = '.') -> list:
    """
    遍历指定路径下的文件和目录。

    :param 路径: 目录路径，默认为当前目录。
    :return: 目录下的文件名和文件夹名列表。
    """
    return os.listdir(路径)

def 文件_遍历指定路径所有子目录(路径: str = '.') -> list:
    """
    遍历指定路径下的所有子目录。

    :param 路径: 要遍历的根目录路径，默认为当前目录。
    :return: 包含三元组(路径, [包含的目录], [包含的文件])的列表。
    """
    return list(os.walk(路径))

def 文件_创建目录(路径: str) -> bool:
    """
    创建单层目录。如果该目录已存在，则抛出异常。

    :param 路径: 要创建的目录路径。
    :return: 创建成功返回True。
    """
    try:
        os.mkdir(路径)
        return True
    except FileExistsError:
        print(f"目录 {路径} 已存在。")
        return False

def 文件_创建多层目录(路径: str) -> bool:
    """
    创建多层目录结构。如果目录已存在，不会抛出异常。

    :param 路径: 要创建的多层目录路径。
    :return: 创建成功返回True。
    """
    try:
        os.makedirs(路径, exist_ok=True)
        return True
    except Exception as e:
        print(f"创建目录 {路径} 失败: {e}")
        return False

def 文件_写到文件(文件名: str, 写入的数据, 模式: int = 0, 编码: str = "utf-8") -> bool:
    """
    将数据写入到文件中。支持文本和二进制写入模式。

    :param 文件名: 要写入数据的文件名。
    :param 写入的数据: 要写入的数据。
    :param 模式: 写入模式，0为文本追加，1为文本覆盖，2为二进制追加，3为二进制覆盖。
    :param 编码: 文件编码，默认为utf-8。仅在文本模式下使用。
    :return: 成功返回True。
    """
    写入模式 = {0: 'a', 1: 'w', 2: 'ab', 3: 'wb'}

    # 确保目录存在
    目录 = os.path.dirname(文件名)
    if 目录 and not os.path.exists(目录):
        os.makedirs(目录)

    # 写入文件
    if 模式 < 2:
        with open(文件名, 写入模式[模式], encoding=编码) as 文件:
            文件.write(写入的数据)
    else:
        with open(文件名, 写入模式[模式], 'wb') as 文件:
            文件.write(写入的数据)

    return True

def 文件_读取文件(文件名: str, 二进制: bool = False, 编码: str = "utf-8", 读取长度: int = -1,**参数):
    """
    从指定文件中读取数据。

    :param 文件名: 要读取的文件名。
    :param 二进制: 是否以二进制模式读取，False表示文本模式，True表示二进制模式。
    :param 编码: 文件编码，默认为utf-8。仅在文本模式下使用。
    :param 读取长度: 要读取的数据长度，-1表示读取全部内容。
    :return: 读取到的数据。
    """
    模式 = 'rb' if 二进制 else 'r'
    打开参数 = {'encoding': 编码} if not 二进制 else {}
    打开参数.update(**参数)
    with open(文件名, 模式, **打开参数) as 文件:
        return 文件.read(读取长度)

def 文件_删除文件(路径: str) -> bool:
    """
    删除指定路径的文件。如果路径是一个目录，则抛出异常。

    :param 路径: 要删除的文件路径。
    :return: 成功删除返回True。
    """
    try:
        os.remove(路径)
        return True
    except IsADirectoryError:
        print(f"删除失败：路径 {路径} 是一个目录，无法删除。")
        return False
    except FileNotFoundError:
        print(f"删除失败：文件 {路径} 不存在。")
        return False

def 文件_删除文件2(路径: str) -> bool:
    """
    删除指定路径的文件。如果路径是一个目录，则抛出异常。

    :param 路径: 要删除的文件路径。
    :return: 成功删除返回True。
    """
    try:
        os.unlink(路径)
        return True
    except IsADirectoryError:
        print(f"删除失败：路径 {路径} 是一个目录，无法删除。")
        return False
    except FileNotFoundError:
        print(f"删除失败：文件 {路径} 不存在。")
        return False

def 文件_删除空目录(路径: str) -> bool:
    """
    删除单层空目录。如果目录非空或删除失败，则返回False。

    :param 路径: 要删除的目录路径。
    :return: 成功删除返回True，否则返回False。
    """
    try:
        os.rmdir(路径)
        return True
    except Exception as e:
        print(f"{sys._getframe().f_code.co_name}|删除目录失败\r\n{traceback.format_exc()}")
        return False

def 文件_删除多层空目录(路径: str) -> bool:
    """
    递归删除多层空目录。从子目录到父目录逐层尝试删除，遇到非空目录则停止。

    :param 路径: 要删除的目录路径。
    :return: 成功删除返回True，遇到非空目录或失败则抛出异常。
    """
    try:
        os.removedirs(路径)
        return True
    except Exception as e:
        print(f"{sys._getframe().f_code.co_name}|删除多层目录失败\r\n{traceback.format_exc()}")
        return False

def 文件_重命名(原文件名: str, 新文件名: str) -> bool:
    """
    重命名文件或文件夹。

    :param 原文件名: 原始的文件或文件夹名称。
    :param 新文件名: 新的文件或文件夹名称。
    :return: 成功返回True。
    """
    os.rename(原文件名, 新文件名)
    return True

def 文件_目录文件名分割(路径: str) -> tuple:
    """
    分割文件路径为目录和文件名。

    :param 路径: 文件的完整路径。
    :return: (目录, 文件名) 的元组。
    """
    return os.path.split(路径)

def 文件_文件扩展名分割(路径: str) -> tuple:
    """
    分割文件路径为文件名和扩展名。

    :param 路径: 文件的完整路径。
    :return: (文件名, 扩展名) 的元组。
    """
    return os.path.splitext(路径)

def 文件_取文件名(路径: str) -> str:
    """
    从文件路径中提取文件名。

    :param 路径: 文件的完整路径。
    :return: 文件名字符串。
    """
    return os.path.basename(路径)

def 文件_取文件目录(路径: str) -> str:
    """
    从文件路径中提取目录路径。

    :param 路径: 文件的完整路径。
    :return: 目录路径字符串。
    """
    return os.path.dirname(路径)

def 文件_更改当前工作目录(路径: str) -> bool:
    """
    更改当前工作目录到指定路径。

    :param 路径: 新的工作目录路径。
    :return: 成功返回True。
    """
    os.chdir(路径)
    return True

def 文件_更改当前进程目录(路径: str) -> bool:
    """
    更改当前进程的根目录到指定路径。这是一个Unix特有的操作，需要管理员权限。

    :param 路径: 新的根目录路径。
    :return: 成功返回True。
    """
    try:
        os.chroot(路径)
        return True
    except PermissionError:
        print(f"权限错误：无法更改进程的根目录到 {路径}")
        return False
    except Exception as e:
        print(f"更改进程根目录时发生错误: {e}")
        return False

def 文件_是否存在(路径: str) -> bool:
    """
    检测指定路径的文件或目录是否存在。

    :param 路径: 要检测的文件或目录的路径。
    :return: 存在返回True，否则返回False。
    """
    return os.access(路径, os.F_OK)

def 文件_是否可读(路径: str) -> bool:
    """
    检测指定路径的文件或目录是否可读。

    :param 路径: 要检测的文件或目录的路径。
    :return: 可读返回True，否则返回False。
    """
    return os.access(路径, os.R_OK)

def 文件_是否可写(路径: str) -> bool:
    """
    检测指定路径的文件或目录是否可写。

    :param 路径: 要检测的文件或目录的路径。
    :return: 可写返回True，否则返回False。
    """
    return os.access(路径, os.W_OK)

def 文件_是否可执行(路径: str) -> bool:
    """
    检测指定路径的文件或目录是否可执行。

    :param 路径: 要检测的文件或目录的路径。
    :return: 可执行返回True，否则返回False。
    """
    return os.access(路径, os.X_OK)

def 文件_是否为绝对路径(路径: str) -> bool:
    """
    判断给定路径是否为绝对路径。

    :param 路径: 要判断的文件或目录的路径。
    :return: 如果路径是绝对路径则返回True，否则返回False。
    """
    return os.path.isabs(路径)

def 文件_是否为目录(路径: str) -> bool:
    """
    判断给定路径是否为目录。

    :param 路径: 要判断的文件或目录的路径。
    :return: 如果路径是目录则返回True，否则返回False。
    """
    return os.path.isdir(路径)

def 文件_是否为文件(路径: str) -> bool:
    """
    判断给定路径是否为文件。

    :param 路径: 要判断的文件或目录的路径。
    :return: 如果路径是文件则返回True，否则返回False。
    """
    return os.path.isfile(路径)

def 文件_是否存在2(路径: str) -> bool:
    """
    判断给定路径是否存在。

    :param 路径: 要判断的文件或目录的路径。
    :return: 如果路径存在则返回True，否则返回False。
    """
    return os.path.exists(路径)

def 文件_取文件大小(路径: str) -> int:
    """
    返回指定文件的大小。

    :param 路径: 文件的路径。
    :return: 文件的大小（字节为单位）。
    """
    return os.path.getsize(路径)

def 文件_获取文件信息(路径: str) -> tuple:
    """
    获取文件的详细信息。

    :param 路径: 文件的路径。
    :return: 文件信息的元组，包括上次访问时间、修改时间和文件大小。
    """
    结果 = os.stat(路径)
    return (结果.st_atime, 结果.st_mtime, 结果.st_size)

def 文件_修改文件时间(路径: str, 时间: tuple) -> bool:
    """
    修改文件的访问时间和修改时间。

    :param 路径: 文件的路径。
    :param 时间: 时间戳元组，格式为(访问时间戳, 修改时间戳)。
    :return: 成功返回True。
    """
    os.utime(路径, 时间)
    return True

def 文件_取最近访问时间(路径: str) -> int:
    """
    获取文件的最后访问时间。

    :param 路径: 文件的路径。
    :return: 最后访问时间的10位时间戳。
    """
    return int(os.path.getatime(路径))

def 文件_取创建时间(路径: str) -> int:
    """
    获取文件的创建时间。

    :param 路径: 文件的路径。
    :return: 文件创建时间的10位时间戳。
    """
    return int(os.path.getctime(路径))

def 文件_取修改时间(路径: str) -> int:
    """
    获取文件的最后修改时间。

    :param 路径: 文件的路径。
    :return: 文件最后修改时间的10位时间戳。
    """
    return int(os.path.getmtime(路径))

def 文件_取文件MD5(路径: str) -> str:
    """
    计算指定文件的MD5哈希值。

    :param 路径: 文件的路径。
    :return: 文件的MD5哈希值（十六进制字符串）。
    """
    with open(路径, mode='rb') as 文件:
        md5_hash = hashlib.md5()
        for 数据块 in iter(partial(文件.read, 128), b''):
            md5_hash.update(数据块)
    return md5_hash.hexdigest()

def 文件_设为只读(路径: str) -> bool:
    """
    将文件或目录设置为只读。

    :param 路径: 文件或目录的路径。
    :return: 成功返回True。
    """
    os.chmod(路径, stat.S_IREAD)
    return True

def 文件_取消只读(路径: str) -> bool:
    """
    取消文件或目录的只读属性。

    :param 路径: 文件或目录的路径。
    :return: 成功返回True。
    """
    os.chmod(路径, stat.S_IWRITE)
    return True

def 文件_设置可执行(路径: str) -> bool:
    """
    将文件或目录设置为可执行。

    :param 路径: 文件或目录的路径。
    :return: 成功返回True。
    """
    os.chmod(路径, stat.S_IEXEC)
    return True

def 文件_允许所有权限(路径: str) -> bool:
    """
    将文件或目录设置为完全权限（可读、可写、可执行）。

    :param 路径: 文件或目录的路径。
    :return: 成功返回True。
    """
    os.chmod(路径, stat.S_IRWXU)
    return True


# -*- coding: utf-8 -*-
import math,random
from decimal import Decimal


def 到数值(数值: float) -> Decimal:
    """
    将浮点数转换为Decimal类型以提高精度。

    :param 数值: 浮点数。
    :return: Decimal类型的数值。
    """
    return Decimal(str(数值))

def 数值_保留位数(数值: float, 位数: int = 2) -> str:
    """
    将数值格式化为字符串，保留指定位数的小数。

    :param 数值: 浮点数。
    :param 位数: 要保留的小数位数。
    :return: 格式化后的字符串。
    """
    return format(数值, '.{}f'.format(位数))

def 数值_取最小数(数值列表: list) -> float:
    """
    从数值列表中找出并返回最小的数值。

    :param 数值列表: 数值的列表。
    :return: 列表中最小的数值。
    """
    return min(数值列表)

def 数值_取最大数(数值列表: list) -> float:
    """
    从数值列表中找出并返回最大的数值。

    :param 数值列表: 数值的列表。
    :return: 列表中最大的数值。
    """
    return max(数值列表)

def 数值_取上入整数(数值: float) -> int:
    """
    将数值向上取整。

    :param 数值: 浮点数。
    :return: 向上取整后的整数。
    """
    return math.ceil(数值)

def 数值_取下入整数(数值: float) -> int:
    """
    将数值向下取整。

    :param 数值: 浮点数。
    :return: 向下取整后的整数。
    """
    return math.floor(数值)

def 数值_四舍五入(数值: float, 保留位数: int = 0) -> float:
    """
    将数值四舍五入到指定位数的小数。

    :param 数值: 浮点数。
    :param 保留位数: 小数点后保留的位数。
    :return: 四舍五入后的数值。
    """
    return round(数值, 保留位数)

def 数值_取绝对值(数值: float) -> float:
    """
    返回数值的绝对值。

    :param 数值: 浮点数。
    :return: 绝对值。
    """
    return abs(数值)

def 数值_求次方(数值: float, 次方数: int) -> float:
    """
    计算数值的指定次方。

    :param 数值: 底数。
    :param 次方数: 指数。
    :return: 次方计算结果。
    """
    return pow(数值, 次方数)

def 数值_是否在范围内(数值: float, 最小值: float, 最大值: float) -> bool:
    """
    判断一个数值是否在指定的范围内。

    :param 数值: 待判断的数值。
    :param 最小值: 范围的最小值。
    :param 最大值: 范围的最大值。
    :return: 如果数值在范围内返回True，否则返回False。
    """
    return 最小值 <= 数值 <= 最大值

def 数值_转换为整数(数值: float) -> int:
    """
    将浮点数转换为整数。

    :param 数值: 浮点数。
    :return: 转换后的整数。
    """
    return int(数值)

def 数值_求和(数值列表: list) -> float:
    """
    计算数值列表的总和。

    :param 数值列表: 包含数值的列表。
    :return: 总和。
    """
    return sum(数值列表)

def 数值_求平均数(数值列表: list) -> float:
    """
    计算数值列表的平均数。

    :param 数值列表: 包含数值的列表。
    :return: 平均数。
    """
    return sum(数值列表) / len(数值列表) if 数值列表 else 0

def 数值_求平方根(数值: float) -> float:
    """
    计算数值的平方根。

    :param 数值: 非负数值。
    :return: 平方根。
    """
    return math.sqrt(数值)

def 数值_取对数(数值: float, 底数: float) -> float:
    """
    计算以指定底数的对数。

    :param 数值: 需要计算对数的数值。
    :param 底数: 对数的底数。
    :return: 对数值。
    """
    return math.log(数值, 底数)

def 数值_取正弦值(角度: float) -> float:
    """
    计算角度的正弦值。

    :param 角度: 角度，以弧度为单位。
    :return: 正弦值。
    """
    return math.sin(角度)

def 数值_取余弦值(角度: float) -> float:
    """
    计算角度的余弦值。

    :param 角度: 角度，以弧度为单位。
    :return: 余弦值。
    """
    return math.cos(角度)

def 数值_取正切值(角度: float) -> float:
    """
    计算角度的正切值。

    :param 角度: 角度，以弧度为单位。
    :return: 正切值。
    """
    return math.tan(角度)

def 数值_弧度转角度(弧度: float) -> float:
    """
    将弧度值转换为角度值。

    :param 弧度: 弧度值。
    :return: 对应的角度值。
    """
    return math.degrees(弧度)

def 数值_角度转弧度(角度: float) -> float:
    """
    将角度值转换为弧度值。

    :param 角度: 角度值。
    :return: 对应的弧度值。
    """
    return math.radians(角度)

def 数值_取指数值(数值: float) -> float:
    """
    计算数值的自然指数值。

    :param 数值: 任意数值。
    :return: e的数值次幂。
    """
    return math.exp(数值)

def 数值_取正弦的逆(数值: float) -> float:
    """
    计算数值的反正弦值。

    :param 数值: -1到1之间的数值。
    :return: 对应的反正弦值（弧度）。
    """
    return math.asin(数值)

def 数值_取余弦的逆(数值: float) -> float:
    """
    计算数值的反余弦值。

    :param 数值: -1到1之间的数值。
    :return: 对应的反余弦值（弧度）。
    """
    return math.acos(数值)

def 数值_取正切的逆(数值: float) -> float:
    """
    计算数值的反正切值。

    :param 数值: 任意数值。
    :return: 对应的反正切值（弧度）。
    """
    return math.atan(数值)

def 数值_取双曲正弦值(数值: float) -> float:
    """
    计算数值的双曲正弦值。

    :param 数值: 任意数值。
    :return: 对应的双曲正弦值。
    """
    return math.sinh(数值)

def 数值_取双曲余弦值(数值: float) -> float:
    """
    计算数值的双曲余弦值。

    :param 数值: 任意数值。
    :return: 对应的双曲余弦值。
    """
    return math.cosh(数值)

def 数值_取双曲正切值(数值: float) -> float:
    """
    计算数值的双曲正切值。

    :param 数值: 任意数值。
    :return: 对应的双曲正切值。
    """
    return math.tanh(数值)

def 数值_求最大公约数(数值1: int, 数值2: int) -> int:
    """
    计算两个整数的最大公约数。

    :param 数值1: 第一个整数。
    :param 数值2: 第二个整数。
    :return: 最大公约数。
    """
    return math.gcd(数值1, 数值2)

def 数值_取余数(被除数: float, 除数: float) -> float:
    """
    计算两个数相除的余数。

    :param 被除数: 被除数。
    :param 除数: 除数。
    :return: 余数。
    """
    return 被除数 % 除数

def 数值_取模(被除数: float, 除数: float) -> float:
    """
    计算两个数相除的模。

    :param 被除数: 被除数。
    :param 除数: 除数。
    :return: 模。
    """
    return math.fmod(被除数, 除数)

def 数值_取平方(数值: float) -> float:
    """
    计算数值的平方。

    :param 数值: 任意数值。
    :return: 平方值。
    """
    return 数值 ** 2

def 数值_取立方(数值: float) -> float:
    """
    计算数值的立方。

    :param 数值: 任意数值。
    :return: 立方值。
    """
    return 数值 ** 3

def 数值_取阶乘(数值: int) -> int:
    """
    计算数值的阶乘。

    :param 数值: 非负整数。
    :return: 阶乘结果。
    """
    return math.factorial(数值)

def 数值_取随机数(最小值: float, 最大值: float) -> float:
    """
    在指定范围内生成一个随机数。

    :param 最小值: 范围的最小值。
    :param 最大值: 范围的最大值。
    :return: 随机数。
    """
    import random
    return random.uniform(最小值, 最大值)

def 数值_是否为素数(数值: int) -> bool:
    """
    判断一个整数是否为素数。

    :param 数值: 待判断的整数。
    :return: 如果是素数返回True，否则返回False。
    """
    if 数值 <= 1:
        return False
    for i in range(2, int(math.sqrt(数值)) + 1):
        if 数值 % i == 0:
            return False
    return True

def 数值_是否为偶数(数值: int) -> bool:
    """
    判断一个整数是否为偶数。

    :param 数值: 整数。
    :return: 如果是偶数返回True，否则返回False。
    """
    return 数值 % 2 == 0

def 数值_是否为奇数(数值: int) -> bool:
    """
    判断一个整数是否为奇数。

    :param 数值: 整数。
    :return: 如果是奇数返回True，否则返回False。
    """
    return 数值 % 2 != 0

def 数值_求和至(数值: int) -> int:
    """
    计算从1加到指定数值的和。

    :param 数值: 正整数。
    :return: 从1加到该数值的和。
    """
    return sum(range(1, 数值 + 1))

def 数值_求积至(数值: int) -> int:
    """
    计算从1乘到指定数值的积。

    :param 数值: 正整数。
    :return: 从1乘到该数值的积。
    """
    产品 = 1
    for i in range(1, 数值 + 1):
        产品 *= i
    return 产品

def 数值_取随机整数(最小值: int, 最大值: int) -> int:
    """
    在指定范围内生成一个随机整数。

    :param 最小值: 范围的最小值。
    :param 最大值: 范围的最大值。
    :return: 随机整数。
    """
    return random.randint(最小值, 最大值)

def 数值_转换为二进制字符串(数值: int) -> str:
    """
    将整数转换为二进制表示的字符串。

    :param 数值: 整数。
    :return: 二进制表示的字符串。
    """
    return bin(数值)

def 数值_转换为十六进制字符串(数值: int) -> str:
    """
    将整数转换为十六进制表示的字符串。

    :param 数值: 整数。
    :return: 十六进制表示的字符串。
    """
    return hex(数值)

def 数值_转换为八进制字符串(数值: int) -> str:
    """
    将整数转换为八进制表示的字符串。

    :param 数值: 整数。
    :return: 八进制表示的字符串。
    """
    return oct(数值)



# -*- coding: utf-8 -*-
import queue,random

class 队列:

    def __init__(self, 队列大小: int = 0, 顺序: bool = True):
        """
        初始化队列。

        :param 队列大小: 队列的最大大小，小于1表示无限长。
        :param 顺序: 为真则队列是先进先出，为假则队列是后进先出。
        """
        if 顺序:
            self.__队列 = queue.Queue(maxsize=队列大小)
        else:
            self.__队列 = queue.LifoQueue(maxsize=队列大小)

    def 是否为空(self) -> bool:
        """
        检查队列是否为空。

        :return: 为空返回True，否则返回False。
        """
        return self.__队列.empty()

    def 是否已满(self) -> bool:
        """
        检查队列是否已满。

        :return: 已满返回True，否则返回False。
        """
        return self.__队列.full()

    def 清空队列(self):
        """
        清空队列中的所有元素。
        """
        while not self.__队列.empty():
            self.__队列.get()

    def 取出成员(self):
        """
        从队列中取出一个元素。

        :return: 取出的元素，如果队列为空，则返回None。
        """
        if not self.__队列.empty():
            return self.__队列.get()
        else:
            return None

    def 取队列成员数(self) -> int:
        """
        获取队列中的元素数量。

        :return: 队列中的元素数量。
        """
        return self.__队列.qsize()

    def 加入成员(self, 值):
        """
        向队列中加入一个元素。

        :param 值: 加入队列的元素。
        """
        self.__队列.put(值)

class 字典类(dict):
    def __init__(self, 初始字典=None):
        """
        初始化字典类的实例。

        :param 初始字典: 用于初始化的字典。
        """
        super().__init__(初始字典 or {})

    def 取值并删除(self, 键, 失败返回值=None):
        """
        如果键存在于字典中，则删除并返回其值；如果不存在，则返回失败返回值。
        :param 键: 要删除的键。
        :param 失败返回值: 如果键不存在时的返回值。
        :return: 删除的键对应的值或失败返回值。
        示例：字典实例.取值并删除('a') # 返回 1
        """
        return self.pop(键, 失败返回值)

    def 取指定键值(self, 键, 失败返回值=None):
        """
        如果键存在于字典中，则返回其值；如果不存在，则返回失败返回值。
        :param 键: 要查找的键。
        :param 失败返回值: 如果键不存在时的返回值。
        :return: 键对应的值或失败返回值。
        示例：字典实例.取指定键值('a') # 返回 1
        """
        return self.get(键, 失败返回值)

    def 清空(self):
        """
        清空字典中的所有项。
        :return: 总是返回True。
        示例：字典实例.清空() # 返回 True
        """
        self.clear()
        return True

    def 拷贝(self):
        """
        创建字典的浅拷贝。
        :return: 字典的一个新的浅拷贝。
        示例：副本 = 字典实例.拷贝() # 创建一个新的字典副本
        """
        return 字典类(self.copy())

    @staticmethod
    def 生成(键值列表, 键值):
        """
        从键列表创建一个新字典，所有键对应的值都是相同的。
        :param 键值列表: 字典键的列表。
        :param 键值: 所有键的共同值。
        :return: 一个新的字典。
        示例：新字典 = 字典类.生成(['a', 'b', 'c'], 1) # 返回 {'a': 1, 'b': 1, 'c': 1}
        """
        return 字典类.fromkeys(键值列表, 键值)

    def 转列表(self):
        """
        将字典的键值对转换为列表。
        :return: 包含字典键值对的列表。
        示例：字典实例.转列表() # 返回 [('a', 1), ('b', 2), ('c', 3)]
        """
        return list(self.items())

    def 取全部键(self):
        """
        返回字典中所有键的列表。
        :return: 字典键的列表。
        示例：字典实例.取全部键() # 返回 ['a', 'b', 'c']
        """
        return list(self.keys())

    def 取全部值(self):
        """
        返回字典中所有值的列表。
        :return: 字典值的列表。
        示例：字典实例.取全部值() # 返回 [1, 2, 3]
        """
        return list(self.values())

    def 取出并删除最后键值(self):
        """
        删除并返回字典中的最后一个键值对。
        :return: 被删除的键值对。
        示例：字典实例.取出并删除最后键值() # 返回 ('c', 3)
        """
        return self.popitem()

    def 取值添加(self, 键, 值=None):
        """
        如果键不存在于字典中，将键值添加到字典。
        :param 键: 要添加的键。
        :param 值: 要添加的值。
        :return: 键对应的值。
        示例：字典实例.取值添加('d', 4) # 返回 4
        """
        return self.setdefault(键, 值)

class 列表类(list):
    def __init__(self, 初始列表=None):
        """
        初始化列表类的实例。

        :param 初始列表: 用于初始化的列表。
        """
        super().__init__(初始列表 or [])

    def 转字典(self):
        """
        将列表转换为字典。
        列表格式应为[(1,2),(3,4)]。
        """
        return {x[0]: x[1] for x in self}

    @staticmethod
    def 合并为字典(列表1, 列表2):
        """
        将两个列表合并为字典。
        例如：[1,2]和[8,9]合并为{1:8, 2:9}。
        """
        return dict(zip(列表1, 列表2))

    def 加入成员(self, 值):
        """
        在列表末尾添加新的对象。
        """
        self.append(值)
        return True

    def 插入成员(self, 位置, 值):
        """
        在指定位置插入一个项目。
        """
        self.insert(位置, 值)
        return True

    def 取出现次数(self, 值):
        """
        返回元素在列表中出现的次数。
        """
        return self.count(值)

    def 合并列表(self, 新列表):
        """
        合并另一个列表或元组到当前列表。
        """
        self.extend(新列表)
        return True

    def 查找成员位置(self, 值):
        """
        返回列表中值的第一个匹配项的索引。
        """
        return self.index(值)

    def 取值并删除(self, 位置=None):
        """
        移除列表中的一个元素（默认最后一个元素），并且返回该元素的值。
        """
        return self.pop(位置) if 位置 is not None else self.pop()

    def 删除指定值(self, 值):
        """
        移除列表中值为 x 的第一个元素。
        """
        self.remove(值)
        return True

    def 倒序排列(self):
        """
        将列表中的元素反向排列。
        """
        self.reverse()
        return True

    def 大小排序(self, 排序方式=False):
        """
        对列表进行排序。
        """
        self.sort(reverse=排序方式)
        return True

def 数组_按成员长度排序(数组):
    """
    根据数组中每个成员的长度进行排序，长度长的在前。

    :param 数组: 要排序的数组。
    :return: 排序后的新数组。
    示例：数组_按成员长度排序(['apple', 'pear', 'banana']) # 返回 ['banana', 'apple', 'pear']
    """
    return sorted(数组, key=lambda i: len(i), reverse=True)


def 数组_按子成员大小排序(数组, 成员索引):
    """
    根据数组中子成员的大小进行排序。

    :param 数组: 包含子数组的数组。
    :param 成员索引: 子数组中用于排序的成员索引。
    :return: 排序后的新数组。
    示例：数组_按子成员大小排序([[1, 'apple'], [3, 'banana'], [2, 'pear']], 0) # 返回 [[3, 'banana'], [2, 'pear'], [1, 'apple']]
    """
    return sorted(数组, key=lambda i: i[成员索引])


def 数组_取随机成员数组(数组, 数量):
    """
    从数组中随机取出指定数量的成员组成新数组。

    :param 数组: 原数组。
    :param 数量: 要取出的成员数量。
    :return: 新数组，包含随机选取的成员。
    示例：数组_取随机成员数组([1, 2, 3, 4, 5], 3) # 可能返回 [3, 1, 5]
    """
    return random.sample(数组, 数量) if 数量 <= len(数组) else False


def 数组_取随机成员(数组):
    """
    从数组中随机取出一个成员。

    :param 数组: 原数组。
    :return: 随机选取的一个成员。
    示例：数组_取随机成员([1, 2, 3, 4, 5]) # 可能返回 3
    """
    return random.choice(数组)


from setuptools import setup



from setuptools import setup, find_packages

setup(
    name='Taowa',
    version='1.0.3',
    description='常用基础命令',
    long_description_content_type='text/markdown',
    author='',
    author_email='',
    packages=find_packages(),

    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    # 如果有依赖包，可以在此处添加
    install_requires=[
        'requests',
        'PyExecjs',
    ],
)


# -*- coding:utf-8 -*-
import threading

def 启动线程(函数名, 参数=[], 跟随主线程结束=False):
    """
    启动一个新的线程来运行指定的函数。

    :param 函数名: 要在新线程中运行的函数。
    :param 参数: 传递给函数的参数列表。
    :param 跟随主线程结束: 如果为True，主线程结束时此线程也会结束。
    :return: 创建的线程对象。
    """
    线程 = threading.Thread(target=函数名, args=参数, daemon=跟随主线程结束)
    线程.start()
    return 线程

class 互斥锁:
    """
    一个互斥锁类，使用上下文管理器确保锁的正确获取和释放。
    可以使用with语句自动管理锁的状态，防止死锁。
    with 锁:
    # 在这里执行需要同步的操作
    # ...
    # 当离开这个块的时候，锁会自动释放
    """
    def __init__(self):
        self.__互斥锁 = threading.Lock()

    def __enter__(self):
        self.__互斥锁.acquire()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.__互斥锁.release()

class 递归锁:
    """
    递归锁类，使用上下文管理器确保锁的正确获取和释放。
    可以使用with语句自动管理锁的状态。
    with 锁:
        # 对共享资源的操作
        共享资源 += 1
        # 递归锁允许在同一线程内重入
        with 锁:
            共享资源 += 1
    """
    def __init__(self):
        self.__递归锁 = threading.RLock()

    def __enter__(self):
        self.__递归锁.acquire()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.__递归锁.release()


class 信号量:
    """
    信号量类，用于限制同时运行的线程数量。
    使用上下文管理器确保信号量的正确获取和释放。
    可以使用with语句自动管理信号量的状态。
    """
    def __init__(self, 数量=1):
        self.__信号量 = threading.BoundedSemaphore(数量)

    def __enter__(self):
        self.__信号量.acquire()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.__信号量.release()

class 事件锁:
    def __init__(self):
        self.__事件锁 = threading.Event()

    def 通行(self):
        self.__事件锁.set()

    def 堵塞(self):
        self.__事件锁.clear()

    def 等待(self):
        self.__事件锁.wait()


class 线程类:
    '''
    线程管理 = 线程类(同时运行线程数=4, 总线程数=20)
    for i in range(100):  # 尝试启动10个线程
        线程 = 线程管理.启动线程(示例任务, 参数=(f"任务{i + 1}",))
    if 线程 is None:
        print(f"达到总线程数限制，线程{i + 1}未启动")
    '''
    def __init__(self, 同时运行线程数=None, 总线程数=None):
        self.__线程列表 = []
        self.__信号量 = threading.Semaphore(同时运行线程数) if 同时运行线程数 else None
        self.__总线程数 = 总线程数
        self.__已启动线程数 = 0

    def 启动线程(self, 函数名, 参数=[], 跟随主线程结束=False):
        if self.__总线程数 is not None and self.__已启动线程数 >= self.__总线程数:
            return None  # 达到总线程数限制，不再创建新线程

        def 线程任务包装器(*args, **kwargs):
            if self.__信号量:
                with self.__信号量:
                    函数名(*args, **kwargs)
            else:
                函数名(*args, **kwargs)

        线程 = threading.Thread(target=线程任务包装器, args=参数, daemon=跟随主线程结束)
        线程.start()
        self.__线程列表.append(线程)
        self.__已启动线程数 += 1
        return 线程

    def 等待线程结束(self, 最长等待时间=None):
        """
        等待所有启动的线程结束。

        :param 最长等待时间: 每个线程的最长等待时间，如果为None，则无限等待。
        :return: 所有线程结束时返回True。
        """
        for 线程 in self.__线程列表:
            线程.join(timeout=最长等待时间)
        return True

    def 取运行中的线程对象(self):
        """
        获取当前运行中的所有线程对象。

        :return: 当前运行中的线程列表。
        """
        return threading.enumerate()

    def 线程是否在运行(self, 线程对象):
        """
        判断指定的线程是否还在运行。

        :param 线程对象: 要检查的线程。
        :return: 如果线程在运行返回True，否则返回False。
        """
        return 线程对象.is_alive()

    def 取运行的线程数(self):
        """
        返回由该类创建并且仍在运行的线程数量。

        :return: 运行中的线程数量。
        """
        return sum(线程.is_alive() for 线程 in self.__线程列表)








# -*- coding: utf-8 -*-
import datetime,time,random,locale



def 时间_取日期_可调整(日期调整: int = 0) -> str:
    """
    根据给定的调整值返回相应的日期。

    :param 日期调整: 日期调整值，正数表示未来的日期，负数表示过去的日期，0表示今天。
    :return: 调整后的日期。
    """
    return str(datetime.date.today() + datetime.timedelta(days=日期调整))

def 时间_取启动时间() -> float:
    """
    返回程序的启动时间，使用高精度计时器。

    :return: 程序启动到当前的时间（秒）。
    """
    return time.perf_counter()

def 时间_取简化星期名称(时间戳 = None) -> str:
    """
    返回本地简化星期名称（如：Mon）。
    示例调用：print(时间_取简化星期名称())
    预期输出：Mon
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%a', time.localtime(时间戳))

def 时间_取完整星期名称(时间戳 = None) -> str:
    """
    返回本地完整星期名称（如：Monday）。
    示例调用：print(时间_取完整星期名称())
    预期输出：Monday
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%A', time.localtime(时间戳))

def 时间_取简化月份名称(时间戳 = None) -> str:
    """
    返回本地简化月份名称（如：Jan）。
    示例调用：print(时间_取简化月份名称())
    预期输出：Jan
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%b', time.localtime(时间戳))

def 时间_取完整月份名称(时间戳 = None) -> str:
    """
    返回本地完整月份名称（如：January）。
    示例调用：print(时间_取完整月份名称())
    预期输出：January
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%B', time.localtime(时间戳))

def 时间_取时间_国外格式(时间戳 = None) -> str:
    """
    返回本地相应的日期和时间表示（如：Mon Jan  1 12:00:00 2023）。
    示例调用：print(时间_取时间_国外格式())
    预期输出：Mon Jan  1 12:00:00 2023
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%c', time.localtime(时间戳))

def 时间_取小时24(时间戳 = None) -> str:
    """
    返回一天中的第几个小时（24小时制，如：12）。
    示例调用：print(时间_取小时24())
    预期输出：12
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%H', time.localtime(时间戳))

def 时间_取小时12(时间戳 = None) -> str:
    """
    返回第几个小时（12小时制，如：01）。
    示例调用：print(时间_取小时12())
    预期输出：01
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%I', time.localtime(时间戳))

def 时间_取分钟(时间戳 = None) -> str:
    """
    返回分钟数（如：00）。
    示例调用：print(时间_取分钟())
    预期输出：00
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%M', time.localtime(时间戳))

def 时间_取秒(时间戳 = None) -> str:
    """
    返回秒数（如：01）。
    示例调用：print(时间_取秒())
    预期输出：01
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%S', time.localtime(时间戳))

def 时间_取本地AM或PM(时间戳 = None) -> str:
    """
    返回本地AM或PM的相应符号（如：AM）。
    示例调用：print(时间_取本地AM或PM())
    预期输出：AM
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%p', time.localtime(时间戳))

def 时间_取一年中的第几天(时间戳 = None) -> str:
    """
    返回一年中的第几天（如：046）。
    示例调用：print(时间_取一年中的第几天())
    预期输出：046
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%j', time.localtime(时间戳))

def 时间_取月份(时间戳 = None) -> str:
    """
    返回月份（如：02）。
    示例调用：print(时间_取月份())
    预期输出：02
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%m', time.localtime(时间戳))

def 时间_取第几星期(时间戳 = None) -> str:
    """
    返回一年中的第几星期数（如：00-53，星期天是一个星期的开始）。
    示例调用：print(时间_取第几星期())
    预期输出：00
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%U', time.localtime(时间戳))

def 时间_取第几星期_周一(时间戳 = None) -> str:
    """
    和%U基本相同，不同的是%W以星期一为一个星期的开始
    示例调用：print(时间_取第几星期_周一())
    预期输出：00
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%U', time.localtime(时间戳))

def 时间_取国外日期格式(时间戳 = None) -> str:
    """
    返回本地相应日期（如：01/01/23）。
    示例调用：print(时间_取国外日期格式())
    预期输出：01/01/23
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%x', time.localtime(时间戳))

def 时间_取时分秒(时间戳 = None) -> str:
    """
    返回本地相应时间（如：12:00:00）。
    示例调用：print(时间_取时分秒())
    预期输出：12:00:00
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%X', time.localtime(时间戳))

def 时间_取年份(时间戳 = None) -> str:
    """
    返回完整的年份（如：2023）。
    示例调用：print(时间_取年份())
    预期输出：2023
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%Y', time.localtime(时间戳))

def 时间_取年份_简写(时间戳 = None) -> str:
    """
    返回去掉世纪的年份（如：23）。
    示例调用：print(时间_取年份_简写())
    预期输出：23
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%y', time.localtime(时间戳))

def 时间_取时区(时间戳 = None) -> str:
    """
    返回时区的名字（如：+0800）。
    示例调用：print(时间_取时区())
    预期输出：+0800
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%z', time.localtime(时间戳))

def 时间_取日期(时间戳 = None) -> str:
    """
    返回日期（如：12-10）。
    示例调用：print(时间_取日期())
    预期输出：+0800
    """
    时间戳 = float(时间戳) if 时间戳 is not None else time.time()
    return time.strftime('%m-%d', time.localtime(时间戳))

def 时间_取现行时间(格式: str = "%Y-%m-%d %H:%M:%S") -> str:
    """
    返回当前时间的字符串表示。

    :param 格式: 时间的格式化字符串，默认为"%Y-%m-%d %H:%M:%S"。
    :return: 格式化后的当前时间字符串。

    示例调用：print(时间_取现行时间())
    预期输出：2023-01-01 12:00:00 （具体输出取决于当前时间）
    """
    return time.strftime(格式, time.localtime())


def 时间_取现行时间戳(是否取十位: bool = False) -> str:
    """
    返回当前时间的时间戳字符串。

    :param 是否取十位: 是否返回10位的时间戳（秒级）。默认为False，返回13位的时间戳（毫秒级）。
    :return: 当前时间的时间戳字符串。
    """
    if 是否取十位:
        return str(round(time.time()))
    else:
        datetime_object = datetime.datetime.now()
        now_timetuple = datetime_object.timetuple()
        now_second = time.mktime(now_timetuple)
        return str(int(now_second * 1000 + datetime_object.microsecond / 1000))


def 时间_取随机时间戳() -> str:
    """
    返回字符串类型的随机时间戳，介于0到1之间的随机浮点数。

    :return: 随机生成的时间戳字符串。
    """
    return str(random.random())

def 时间_时间转时间戳(时间: str, 格式: str = "%Y-%m-%d %H:%M:%S") -> str:
    """
    将符合特定格式的时间字符串转换为10位的时间戳（秒级）。

    :param 时间: 时间字符串。
    :param 格式: 时间的格式，默认为"%Y-%m-%d %H:%M:%S"。
    :return: 字符串类型的10位时间戳。

    示例调用：print(时间_时间转时间戳("2023-01-01 12:00:00"))
    预期输出：'1672531200'
    """
    locale.setlocale(locale.LC_ALL, '')
    return str(int(time.mktime(time.strptime(时间, 格式))))

def 时间_时间戳转时间(时间戳, 格式: str = '%Y-%m-%d %H:%M:%S') -> str:
    """
    将10位或13位的时间戳转换为格式化的时间字符串。

    :param 时间戳: 可以是10位或13位的字符串或浮点数时间戳。
    :param 格式: 时间的格式，默认为'%Y-%m-%d %H:%M:%S'。
    :return: 格式化的时间字符串。
    """
    时间戳 = float(时间戳)  # 转换为浮点数以处理字符串和浮点数输入
    if 时间戳 >= 1e12:  # 检测是否为13位时间戳
        时间戳 /= 1000  # 转换为秒
    return time.strftime(格式, time.localtime(int(时间戳)))


def 时间_取时间间隔(原时间: str, 对比时间: str) -> int:
    """
    计算两个时间字符串之间的时间差（秒数）。

    :param 原时间: 起始时间字符串，格式应为"%Y-%m-%d %H:%M:%S"。
    :param 对比时间: 结束时间字符串，格式应为"%Y-%m-%d %H:%M:%S"。
    :return: 两个时间之间的差值（秒数），整数型。

    示例调用：print(时间_取时间间隔("2023-01-02 12:00:00", "2023-01-01 12:00:00"))
    预期输出：86400 （表示两个时间相差一天，即24小时*60分钟*60秒）
    """
    return int(time.mktime(time.strptime(原时间, "%Y-%m-%d %H:%M:%S"))) - int(
        time.mktime(time.strptime(对比时间, "%Y-%m-%d %H:%M:%S")))

def 时间_增减(原时间, 增减部分, 增减数值):
    """
    对原始时间进行增加或减少操作。

    :param 原时间: 可以是字符串或datetime对象。
    :param 增减部分: 1-星期, 2-天, 3-小时, 4-分钟, 5-秒, 6-毫秒。
    :param 增减数值: 增加或减少的数值，正数表示增加，负数表示减少。
    :return: 操作后的datetime对象。

    示例调用及预期输出：
    - print(时间_增减("2023-01-01 12:00:00", 2, 5))  # 增加5天
      输出：2023-01-06 12:00:00
    - print(时间_增减(datetime.datetime.now(), 4, -30))  # 减少30分钟
      输出：当前时间减少30分钟后的时间
    """
    时间差映射 = {
        1: datetime.timedelta(weeks=增减数值),
        2: datetime.timedelta(days=增减数值),
        3: datetime.timedelta(hours=增减数值),
        4: datetime.timedelta(minutes=增减数值),
        5: datetime.timedelta(seconds=增减数值),
        6: datetime.timedelta(milliseconds=增减数值)
    }
    时间差 = 时间差映射[增减部分]
    if isinstance(原时间, str):
        locale.setlocale(locale.LC_ALL, '')
        原时间 = datetime.datetime.strptime(原时间, "%Y-%m-%d %H:%M:%S")
    return 原时间 + 时间差


def 时间_文本转时间(时间文本: str, 格式: str = '%Y-%m-%d %H:%M:%S') -> datetime.datetime:
    """
    将字符串格式的时间转换为 datetime 对象。

    :param 时间文本: 字符串形式的时间。
    :param 格式: 时间文本的格式，默认为 '%Y-%m-%d %H:%M:%S'。
    :return: datetime 对象。
    """
    return datetime.datetime.strptime(时间文本, 格式)


def 时间_时间转文本(时间: datetime.datetime, 格式: str = "%Y-%m-%d %H:%M:%S") -> str:
    """
    将datetime时间对象转换为指定格式的字符串。

    :param 时间: datetime时间对象。
    :param 格式: 时间的格式化字符串，默认为"%Y-%m-%d %H:%M:%S"。
    :return: 格式化后的时间字符串。
    """
    return 时间.strftime(格式)


def 时间_格式化(原时间, 时间格式: str = "%Y-%m-%d %H:%M:%S") -> str:
    """
    将原始时间（可以是字符串或datetime对象）格式化为指定格式的字符串。

    :param 原时间: 原始时间，可以是符合特定格式的字符串或datetime对象。
    :param 时间格式: 目标时间格式的字符串，默认为"%Y-%m-%d %H:%M:%S"。
    :return: 格式化后的时间字符串。
    """
    if isinstance(原时间, str):
        return datetime.datetime.strptime(原时间, "%Y-%m-%d %H:%M:%S").strftime(时间格式)
    else:
        return 原时间.strftime(时间格式)

